/**************************************************************

   Program Title:  P1 AVR Boot/Monitor
   
   Program Name:  P1_AVR_Utilities.c	Version: 0.87
   
   Author:  SS	  					  Date:  August 10, 2005
   
   Compiled with ImageCraft ICCAVR Standard V7.02
   
   This file contains routines that implement a set of basic utility 
   functions for the P1 AVR Boot Monitor.
   
   Fuse settings:  BOOTSZ1, BOOTSZ0, SUT1, CKSEL3, CKSEL1, CKSEL0
   
   Copyright 2004-2005, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright (C) 2005 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Monitor.c,v $
 * Revision 0.55  2005/07/17 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom32v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <stdio.h>
#include "P1_AVR_Uart.h"
#include "P1_AVR_Defines.h"
#include "P1_AVR_Extern.h"

//*********************************************************
//*********************************************************
//      Utilities
//*********************************************************
//*********************************************************
void Delay(void)
{
	unsigned char a, b;

	for (a = 1; a; a++)
		for (b = 1; b; b++)
			;
}

void shortdelay(int cnt)
{
 	unsigned char inbyte;
	int i;
	
    for(i=0; i<cnt; i++)  {
	    inbyte = 0x55;
		}
}

void LongDelay(int cnt)
{
 	 while(cnt > 0) {
	 	 Delay();
		 cnt--;
		 }
}

void Delaymsec(int delayvalue)
{
 	 mseccountdown2 = delayvalue;
	 while(mseccountdown2) ;
}

void GREEN_LED(unsigned char mode)
{
 	if(mode==ON) PORTD &= ~BIT(6);	// low output to turn LED on
	else PORTD |= BIT(6);	// low output to turn LED off
}

void RED_LED(unsigned char mode)
{
 	 if(mode==ON) PORTD &= ~BIT(7);	
	 else PORTD |= BIT(7);	
}

// Function to initiate an EEPROM write
// writes the specified data byte to the specified location
// this will fail if the EEPROM is not ready!
void RTEEPROMwrite(int location, unsigned char databyte)
{
	unsigned char savedSREG;
	EEAR = location;								// set address
	EEDR = databyte;								// set data	
	savedSREG = SREG;								// keep setting so it can be restored
	CLI();													// disable interrupts
	EECR |= BIT(EEMWE);							// set "write enable" bit
	EECR |= BIT(EEWE);							// set "write" bit
	SREG = savedSREG;								// restore SREG
	EEAR = 0;
}

// EEPROM writes take about 8ms - so poll until done
// Do not use ICC V7 function EEPROMwrite() - this routine turns interrupts off!
void EEPROMWrite(int location, unsigned char databyte)
{
 	 while(!RTEEPROMReady())  ;
 	 RTEEPROMwrite(location,databyte);
 	 while(!RTEEPROMReady())  ;
}

// Write a string of characters to EEPROM at specified start location
void WriteEEPROM(int location, unsigned char *strg)
{
 	 int i;
	 
	 i = 0;
	 while(strg[i] != 0x00)  {
	     EEPROMWrite(location+i,strg[i]);
		 i++;
		 }
}


//  Reset the Blackfin for a short period
void ShortBlackfinReset(void)
{
 	 PORTD &= 0xF7;
 	 Delay();
	 PORTD |= 0x08;
}

// Halt Blackfin - Reset line is on Port D pin 3
void HaltBlackfin(char mode)
{
 	RED_LED(ON);
	PORTD &= 0xF7;
	if(mode!=QUIET)  puts("Halt Blackfin\n\r");
}

void ReleaseBlackfin(char mode)
{
 	RED_LED(OFF);
	PORTD |= 0x08;
	if(mode!=QUIET)  puts("Release Blackfin\n\r");
}

// Blackfin Reset line is on Port D pin 3
void ResetBlackfin(char mode)
{
 	RED_LED(ON);
	PORTD &= 0xF7;
	LongDelay(1);
	PORTD |= 0x08;
	RED_LED(OFF);
	if(mode!=QUIET)  puts("Blackfin Reset and enabled\n\r");
}

void BlinkGreen(int period)
{
 	 GREEN_LED(ON);
	 LongDelay(period);
	 GREEN_LED(OFF);
	 LongDelay(period);
}

void BlinkRed(int period)
{
 	 RED_LED(ON);
	 LongDelay(period);
	 RED_LED(OFF);
	 LongDelay(period);
}

// Select Blackfin boot mode - 00, 01, 10 or 11
//
// Blackfin Rev 0.3 silicon boots from Atmel SPI Flash as Boot Mode 3.
// Note: Rev 0.2 silicon can not communicate with Atmel SPI Flash
//       devices AT45DB041, AT45DB081 or ATDB45161 - must use SPI Flash
//       that supports read command 0x03 such as the Microship 25LC640
//       or ST M25P80.
void BlackfinBootMode(unsigned char bootmode)
{
 	 //  Boot mode 0 - parallel flash
 	 if(bootmode==BOOTMODE0) PORTD &= 0xCF;
	 
	 //  Blackfin SPI Slave to AVR
	 else if(bootmode==BOOTMODE2) {
	     PORTD &= 0xDF;
		 PORTD |= 0x10;
		 }
	 //  Boot from SPI Slave Flash
	 //     Rev 0.2 silicon uses read command 0x03 
	 //     Rev 0.3 silicon attempts 0x03 read then switches to Atmel SPI
	 else if(bootmode==BOOTMODE3) PORTD |= 0x30;
}

//  Fast blackfin boot mode setting function
void ResetBlackfinBootMode(unsigned char bootmode)
{
 	 HaltBlackfin(QUIET);
	 shortdelay(100);
 	 BlackfinBootMode(bootmode);
	 shortdelay(100);	 
	 //ReleaseBlackfin(QUIET);
}

void PrintHex(unsigned char inbyte)
{
 	 bytetohex(inbyte);
	 puts(hexbyte);
}

void PrintWordHex(unsigned int inword)
{
 	 addresstohex(inword);
	 puts(hexaddress);
}

void PrintLongHex(long address,unsigned char nobytes)
{
 	 unsigned char i,outbyte,nonzerobyte;
	 long tempaddress;
	 
	 nonzerobyte = NO;
	 for(i=0; i<nobytes; i++) {
	     tempaddress = address;
		 tempaddress = tempaddress>>(((nobytes-1)*8)-8*i);
	     outbyte = tempaddress&0x00000000000000ff;
		 if(outbyte==0) {
		 	if(nonzerobyte==YES) {
		  	    bytetohex(outbyte);
	 	        puts(hexbyte); 
			    }
			}
		 else {
		  	 bytetohex(outbyte);
			 if((nonzerobyte==NO)&&(hexbyte[0]=='0')) putchar(hexbyte[1]);
	 	     else puts(hexbyte); 
		     nonzerobyte = YES;
			 }
		 }
}

//  TBD
void EEPROMSetup(void)
{
 	
}

//  Access contents of AVR flash at specified address
unsigned char ReadAVRFlash(int address)
{
 	unsigned char flashbyte,rbyte;
	
	flashbyte = 0;
 	asm("mov r30,r16 \n"
       "mov r31,r17 \n"
	   "lpm r0,Z \n"
	   "mov %flashbyte,r0 \n"); 
	rbyte = flashbyte;  
	return(rbyte);                              
}

//  Output program version
//   TBD - read from EEPROM
void ProgramVersion(void)
{
      char version[5]="0.87";
	  
	  putfs(progmessage);
	  puts(version); 
	  puts("\n\rF1-");
	  DisplayFlashType(flashtype0); puts(", F2-");
	  DisplayFlashType(flashtype1); puts(", Cb-");
	  DisplayCalibrationByte(); puts(", 16K");
	  putCRLF();
}

//  Output sign on message loaded from Flash
void signon()
{
    putCRLF(); putCRLF();
	//putfs(signon1);
	ProgramVersion();
	putfs(signon2);	
	putfs(signon3);
	putfs(signon4);
	putfs(signon5);
	putfs(signon6);
	putfs(signon63);
	putfs(signon65);
	putfs(signon82);
	putfs(signon83);
	putfs(signon85);
	putfs(signon7);
	putfs(signon8);
	putfs(signon9);
	putfs(crlfbar);
}


//  Display AVR EEPROM memory
void DisplayEEPROM(long address,long numbytes)
{
 	 unsigned char i,byte,bytesonlastline,ascii[16],asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	     crcvalue = 0;
	     putchar(':');
		 PrintHex(bytesonline);		 //  Output bytes on line
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = EEPROMread(i+newaddress);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
		 PrintHex((~crcvalue)+0x01);
	     if(showascii==YES)  {
	         puts("  ");
		     for(i=0; i<bytesonline; i++) {
		         if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			     else putchar('.');
			     }
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 PrintHex(bytesonlastline);
		 crcvalue += bytesonlastline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     addresstohex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = EEPROMread(i+address+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 // Output CRC
		 PrintHex((~crcvalue)+0x01);
	     if(showascii==YES)  {
	         puts("  ");
		     for(i=0; i<bytesonlastline; i++) {
		         if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			     else putchar('.');
			     }
			 }
	     putCRLF();  
		 }
	 puts(":00000001FF\n\r");
}

//  Display P1 AVR Flash memory
void DisplayAVRFlash(long address,long numbytes)
{
 	 unsigned char i,byte,bytesonlastline,ascii[16],asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	 	 crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 PrintHex(bytesonline);
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     if(showascii==YES)  {
	         puts("  ");
		     for(i=0; i<bytesonline; i++) {
		         if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			     else putchar('.');
			     }
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 PrintHex(bytesonlastline);
	     //  Output hex start address
	     PrintWordHex(address+linecnt*16);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     if(showascii==YES)  {
		 	 puts("  ");
		     for(i=0; i<bytesonlastline; i++) {
		         if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			     else putchar('.');
			     }
			 }
	     putCRLF();  
		 }
	 puts(":00000001FF\n\r");
}

unsigned char hextobyte(unsigned char inbyte)
{
	 if((inbyte >= 'A')&(inbyte<='F')) return (inbyte - 'A' + 10);
	 if((inbyte >= 'a')&(inbyte<='f')) return (inbyte - 'a' + 10);
	 else return (inbyte - '0');
}

// Read in a hex string and convert to int or long
//  TBD - add long support for serial flash support
long  ScanHex()
{
 	 long  address;
 	 char i=0,len;
	 
	 address = 0;
	 putchar((instrg[0]=getchar()));
 	 while((instrg[i]!=CR)&&(instrg[i]!=' ')) {
	 	 i++;
	     putchar((instrg[i]=getchar()));
		 }
	 instrg[i] = 0;
	 len = strlen(instrg);
	 address = hextobyte(instrg[len-1]);
	 i = 1;
	 while(i<len) {
	     if(i==1) address += ((long) hextobyte(instrg[len-i-1])*16);
	     if(i==2) address += ((long) hextobyte(instrg[len-i-1])*256);
	     if(i==3) address += ((long) hextobyte(instrg[len-i-1])*4096);
	     if(i==4) address += ((long) hextobyte(instrg[len-i-1])*65536);
	     if(i==5) address += ((long) hextobyte(instrg[len-i-1])*1048576);
	     if(i==6) address += ((long) hextobyte(instrg[len-i-1])*16777216);
		 i++;
		 }
	 return address;
}

unsigned char GetByteHex()
{
 	 char strg[3];
	 
	 putchar((strg[0]=getchar()));
	 putchar((strg[1]=getchar()));
	 strg[2] = 0;
	 return converttohex(strg);
}

void DisplayMemory(void)
{
    unsigned char inchar;
	long address, length;
	
	inchar = getchar();
	putchar(inchar);
	putCRLF();
	putfs(displaymessage1);
	address = ScanHex(); 
	length = ScanHex();
	putCRLF();
	switch (inchar) {
	    case 'e': case 'E':
			DisplayEEPROM(address,length);
		    break;
		case 'a': case 'A':
		    DisplayAVRFlash(address,length);
		    break;
		//  In order to access Blackfin Flash the AVR must ask the Blackfin
		//  to release control of the SPI lines.  This is simulated in Rev A
		//  boards by resetting the Blackfin (which should release the SPI lines)
		//  On completion the AVR SPI lines are released.
		case 'f': case 'F':
			//  Reset Blackfin to boot mode 0 - release reset
			ResetBlackfinBootMode(BOOTMODE0);

			Init_SPI_Master();
		    DisplaySerialFlash(address,length,READCONTINUOUS);			
			Release_SPI();
			
			//  Reset Blackfin to boot mode 3
			ResetBlackfinBootMode(BOOTMODE3);
			ReleaseBlackfin(QUIET);
		    break;
		case 'b': case 'B':
			//  Reset Blackfin to boot mode 0 - release reset
			ResetBlackfinBootMode(BOOTMODE0);

			Init_SPI_Master();
		    DisplaySerialFlash(address,length,READBUFFER);				
			Release_SPI();
			
			//  Reset Blackfin to boot mode 3
			ResetBlackfinBootMode(BOOTMODE3);
		    break;
		default:
		    putfs(displaymessage33);
		    break;
	    }
}


// Check intelbinary CRC 
int checkcrc(unsigned char cycbyte)
{
 	unsigned char crcbyte;
	
	GetByteHex();
 	return 1;
}

// Convert Intel Hex ASCII string to binary
//
// :10 0100 00 0C948F0D0C941D0E0C941D0E0C941D0E 38
// :1001000000C941D0E0C941D0E0C944E590C941D0E38
// This routine decodes a line of Intel Hex.  Other 
// routines write the data to either AVR Flash, external Flash
// or AVR EEPROM
//
//   :02 0000 04 0000FA  - IntelHex32 extended address record
// 
//  TBD - check crc byte - drop record if incorrect
int DecodeIntelHex(void)
{
 	 unsigned char strg[8],addressbyte,inbyte;
	 int i;
	 
	 putchar(':');
	 // Convert count of bytes on line
	 bytesonline = GetByteHex();
	 
	 // Convert address to binary - ignore filler byte
	 // Typical Intel Hex consists of 4 ASCII characters
	 inteladdress = GetByteHex()*256;
	 inteladdress += GetByteHex();
	 
	 // Read record type byte (2 characters)
	 intelrecordcommand = GetByteHex();
	 
	 if(intelrecordcommand==0x04) {
	     intelextendedaddress = GetByteHex()*256;
		 intelextendedaddress += GetByteHex();
		 }
	 else  {
	     // Convert all data to binary
	     for(i=0; i<bytesonline; i++) {
		     inbyte = GetByteHex(strg);
		     intelbinary[i] = inbyte;
	 	     }
		 }
		 
	// Check CRC - todo
	if(checkcrc(inbyte)) return 1;
	else return -1;
}

// function to test if the EEPROM is ready for a read or write operation
// returns non zero if ready, zero if not ready 
unsigned char RTEEPROMReady(void)
{
	return !(EECR & 0x02);
}

//  Write decoded Intel hex to EEPROM
void ProgramEEPROM(void)
{
     unsigned char i;
	 
	 for(i=0; i<bytesonline; i++)  EEPROMWrite(inteladdress+i,intelbinary[i]);
}

//  TBD - AVR Flash hold program so programming AVR flash may overwrite program
void ProgramAVRFlash(void)
{
 	 char inchar;
	 
 	 while((inchar=getchar())!=CR) putchar(inchar);
	 putCRLF(); putchar('-');
}

void CleanUpFlashState()
{
	if(flashdatainbuffer==YES)  {
	    //inteladdress = lastaddress;
		//puts("clean ");  PrintLongHex(lastaddress,6);
	    ProgramBuffertoPageErase(lastaddress,1);
		WaitForFlashFree(1);
		intelextendedaddress = 0;
		}
	if(flashprogrammingactive==YES)  {
		Release_SPI();
		flashprogrammingactive = NO;
		flashpageloaded = NO;
		flashpageboundarytraversed = NO;
		flashdatainbuffer = NO;
		}
	//  Restore Blackfin to normal operation - boot mode 3
	ResetBlackfinBootMode(BOOTMODE3);
	ReleaseBlackfin(QUIET);
}

//  Main memory programming selection function
//  Program EEPROM, AVR Flash or SPI Flash.
//  Three IntelHex32 Records 0x00, 0x01 and 0x04 are processed by this
//  routine.  0x00 is a data record, 0x01 is End of File record and 
//  0x04 is extended address record
//
//  TBD - no intel record time out - reset defaults if no intel 
//        records received after 3 seconds
void ProgramMemory(void)
{
 	DecodeIntelHex();
	switch (programmode) {
	    case LEEPROM:
			if(intelrecordcommand==0) ProgramEEPROM();
			break;
		case LAVRFLASH:
		    if(intelrecordcommand==0) ProgramAVRFlash();
			break;
			
		//  To access Blackfin Flash the Blackfin must not have control 
		//  of the shared SPI lines and the Blackfin reset line must not
		//  be active.  The Blackfin releases control of the SPI lines under 
		//  software control or by being forced to boot mode 0.  If the Blackfin
		//  is reset to boot mode 3 it will attempt to load a program from 
		//  serial flash preventing the AVR from accessing the serial flash.
		
		//  This implementation forces the Blackfin to reboot.
		//  Cooperative flash access still TBD
		case LFLASH:
		    // Data record			
			if(intelrecordcommand==0) {
			    flashcountdown = 3000;  // 3 second no new Hex record timeout
			    //  See is SPI Master is active - set active if not
			    if(flashprogrammingactive==NO) {
				    // Reset Blackfin to boot mode 0  forcing SPI lines release
			        ResetBlackfinBootMode(BOOTMODE0);
				    Init_SPI_Master();
					}
					
		        ProgramSerialFlash();		
				}
			// Extended address record
			else if(intelrecordcommand==0x04) ; // nothing to do
			
			// End of File record - reset all variables
			else if(intelrecordcommand==0x01) CleanUpFlashState();
			break;
		} 
}

//  Set default memory programming mode
//  Subsequent Intel Hex dumps will load to the specific memory
void LoadMemory(void)
{
    unsigned char inchar;
	
	inchar = getchar();
	putchar(inchar);
	switch (inchar) {
	    case 'e': case 'E':
			programmode = LEEPROM;
			putfs(loadmessage1);
		    break;
		case 'f': case 'F':
			programmode = LFLASH;
			putfs(loadmessage2);
		    break;
		case 'a': case 'A':
			programmode = LAVRFLASH;
		    putfs(loadmessage3);
		    break;
		default:
		   break;
	    }
}

void DisplayFlashType(unsigned char flashtype)
{
      if(flashtype==AT45DB041ID) putfs(signon121);
      else if(flashtype==AT45DB081ID) putfs(signon122);  
      else if(flashtype==AT45DB161ID) putfs(signon123);  
      else if(flashtype==AT45DB321ID) putfs(signon124);  
      else if(flashtype==AT45DB642ID) putfs(signon125); 
	  else putfs(signon126); 
}

void DisplayCalibrationByte()
{
 	 bytetohex(OSCCAL);
	 puts(hexbyte);
}

//  Read selected analog channel - wait until it completes
//   TBD - background interrupt process
int AnalogCh(int ch)
{
 	unsigned char low,high;
	
	
	if(ch==0) ADMUX = 0x40;  // Select AVCC, ch 0
	else if(ch==1) ADMUX = 0x41; // Select AVCC, ch 0
	else if(ch==2) ADMUX = 0x42;
	else if(ch==3) ADMUX = 0x43;
	else if(ch==4) ADMUX = 0x44;
	else if(ch==5) ADMUX = 0x45;
	else if(ch==6) ADMUX = 0x46;
	else if(ch==7) ADMUX = 0x47;
	
	ADCSRA = 0xC1; // Enable ADC, start conversion, max clock
	
	while(!(ADCSRA&0x10)) ; // loop until conversion completes
	
	low = ADCL;
	high = ADCH;
 	return (low+high*256);
}

void DumpAnalogChs(void)
{
    int  i,analogvalue;
	
	puts("Analog Channels 0..7\n\r");
 	for(i=0; i<8; i++) {
	    analogvalue = AnalogCh(i);
		printint(analogvalue,4,' ');
		} 
	puts("\n\r"); 
}

char getint()
{
	char intvalue,inchar;
	
	inchar = getchar();
	putchar(inchar);
	if(inchar==CR) return 0x78;
	intvalue = (inchar - '0')*10;
	inchar = getchar();
	putchar(inchar);	
	intvalue = intvalue + (inchar-'0');
	return intvalue;
}

// Update the current day and time
void  getsettime()
{
    unsigned char instrg[4],inchar;
	
    _CLI();
	instrg[0] = days;
	instrg[1] = hours;
	instrg[2] = minutes;
	instrg[3] = seconds;
	_SEI();	
	
    puts("Set time [hh:mm:ss]=");
	printint(instrg[0],3,'0'); putchar(' ');
	printint(instrg[1],2,'0'); putchar(':');
	printint(instrg[2],2,'0'); putchar(':');
	printint(instrg[3],2,'0'); puts(" :");
	
	if((instrg[1]=getint())==0x78) return;
	inchar = getchar(); putchar(inchar);
	instrg[2] = getint();
	inchar = getchar(); putchar(inchar);
	instrg[3] = getint();
	inchar = getchar(); putchar(inchar);

    _CLI();
	hours = instrg[1];
	minutes = instrg[2];
	seconds = instrg[3];
	_SEI();	
}

//  Convert ASCII ip address to binary - TBD
void convertiptobinary(unsigned char *ip)
{
}

//  Get set Internet settings
void InternetSettings(void)
{
	convertiptobinary(IPADDRESS);
}

//  Simple test routine
void TestAVR(void)
{
 	 char i,j;
	 
 	 puts("\n\rTesting AVR Version 0.34\n\r");
//	 TestAVRRAM();
//	 TestAVREEPROM();
//	 TestAVRFLASH();
//	 TestBoardFLASH():
	 j = 0;
	 putchar(' ');
	 for(i=0x20; i<0x7f; i++) {
	 	 putchar(i);
		 if(j++ == 60)  {
		     j = 0;
			 puts("\n\r ");
			 }
		 }
	 puts("\n\r ");
	 DumpAnalogChs();
}

//  Set Blackfin boot mode
void SetBlackfinBootMode(void)
{
 	unsigned char bootmode;
	
    putfs(signon66);  // Ask for boot mode
	bootmode = getchar();
	putchar(bootmode);
	putfs(signon67);
	putchar(bootmode);
	puts("\n\r");
	BlackfinBootMode(bootmode-0x30);
}

int Calibrate()
{
	 rtccountdown1 = 128;
	 mseccountdown3 = 750;
	 while(rtccountdown1) ;
	 return mseccountdown3;
}

//  Calibrate Internal Oscillator
//    Check for value in EEPROM first, then calculate new value
//    Compare new value to stored value (if any) and request 
//    store permission
void CalibrateOsc(unsigned char mode)
{
 	 char inchar;
 	 int  timedelay,attempts;
	 
 	 if(mode==VERBOSE) {
	     putfs(calmessage1);
	     DisplayCalibrationByte();
		 }
	 
	 //  Setup one second delay
	 //  Difference between mseccounddown3 and 1000 gives percentage
	 //  offset of internal osc.
	 attempts = 20;
	 while(attempts--)  {
	     timedelay = Calibrate();
	     //printint(timedelay,4,' ');
	     if((256-timedelay)>0)  OSCCAL -= 1;
	     else  OSCCAL += 1;
		 if(abs(256-timedelay)<4) break;
		 }
 	 if(mode==VERBOSE) {
	 	 putCRLF();
	     putfs(calmessage2);
	     DisplayCalibrationByte();
	 	 putCRLF();
		 putfs(calmessage3);
		 inchar = getchar();
		 if((inchar=='y')||(inchar=='Y')) EEPROMwrite(19,OSCCAL);  
	 	 putCRLF();
		 }
}

void SetNewBaudRate(unsigned char baudchar)
{
	  if(baudchar=='2')       ChangeBaudRate(BAUDRATE2400);
	  else if(baudchar=='4')  ChangeBaudRate(BAUDRATE4800);
	  else if(baudchar=='9')  ChangeBaudRate(BAUDRATE9600);
	  else if(baudchar=='1')  ChangeBaudRate(BAUDRATE19200);
	  else if(baudchar=='3')  ChangeBaudRate(BAUDRATE38400);
	  else if(baudchar=='5')  ChangeBaudRate(BAUDRATE57600);
}

void EnterNewBaudRate(void)
{
      unsigned char i,firstchar,inchar;
	  
	  putCRLF();
      putfs(changebaudmessage);
	  firstchar = getchar();
	  putchar(firstchar);
	  while((inchar=getchar())!=CR) putchar(inchar) ;
	  putCRLF();
	  putfs(calmessage3);
	  inchar = getchar();
	  putchar(inchar);
	  if((inchar=='y')||(inchar=='Y')) EEPROMwrite(20,firstchar); 
	  SetNewBaudRate(firstchar);
	  //putCRLF();
}
