/**************************************************************

   Program Title:  P1 AVR Boot/Monitor
   
   Program Name:  P1_AVR_Init.c		  Version: 0.87
   
   Author:  SS	  					  Date:  August 10, 2005
   
   Compiled with ImageCraft ICCAVR Standard V7.02
   
   This file contains routines that implement the basic initialization 
   routines for the P1 AVR Boot Monitor.
   
   Copyright 2004-2005, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright (C) 2005 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Monitor.c,v $
 * Revision 0.55  2005/07/17 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom16v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <eeprom.h>
//#include <stdio.h>
#include "P1_AVR_Uart.h"
#include "P1_AVR_Defines.h"
#include "P1_AVR_Extern.h"

//****************************************************************
//****************************************************************
// Initialization code
//****************************************************************
//****************************************************************

// Initialize UART
void InitUART(unsigned char baudrate)
{
	unsigned char x;
	
	// Set the baud rate
	UBRRH = 0x00;
	UBRRL = baudrate; 
	
	// Enable UART receiver and transmitter
	UCSRB = BIT(RXCIE)|BIT(UDRIE)|BIT(RXEN)|BIT(TXEN); 
    //  8 data bits, 2 stop bits
//	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
    //  8 data bits, 1 stop bit
	UCSRC = (1<<URSEL)|(3<<UCSZ0);

	x = 0; // flush receive buffer
	rxcnt = x; // rx buffer empty
	xoffsent = NO;
	xonsent = NO;
	checkflag = NO;
	enablexonoff = NO; // turn serial driver xon/xoff flow control on
	echoinchars = NO;   // echo all received characters
	UART_RxTail = x;
	UART_RxHead = x;
	UART_TxTail = x;
	UART_TxHead = x;
}

//  Setup the Analog to Digital converter
//  TBD - convert to background interrupt process
void InitADC(void)
{
	DDRA = 0x00;  // All inputs
		   		  // Setup Analog TDB
}

//  Start the real time clock 
//  Clocked by external 32.768KHz watch crystal
//  Set prescalar to 0, set to interrupt using compare match 
//  set to 128 (4msec interrupt rate).  
void InitRealTimeClock(void)
{  
    // Software counters - approximately 4msec resolution
	rtccountdown1 = 0;  
	rtccountdown2 = 0;
	rtccountdown3 = 0;
	rtccountdown4 = 0;
	rtcclockcnt = 0;
	seconds = 0;
	minutes = 0;
	hours = 0;
	days = 0;
	
 	TCNT2 = 0;
 	// Select external real time clock oscillator
	ASSR = (1<<AS2);
	// Set match interrupt to 4msec
	OCR2 = 128;
	// Reload counter on match interrupt - pre-scalar set to 0
	TCCR2 = (1<<WGM21)|(1<<CS20);   
    // setup interrupts for real time clock
    TIMSK |= (1<<OCIE2);
}

// Timer2 is called every 1msec - used to maintain four count down 
// software timers to control precise time delays in various routines
void Init_Timer0(void)
{
 	 mseccountdown1 = 0;
	 mseccountdown2 = 0;
	 mseccountdown3 = 0;
	 flashcountdown = 0;
	 
	 TCNT0 = 0;
	 // Set mode 2 - reset counter on compare match, clk/8
	 TCCR0 = (1<<WGM01)|(1<<CS01)|(1<<CS00);
	 // Reset count after 1 msec
	 OCR0 = 125;
	 // Enable compare match interrupt vector
	 TIMSK |= (1<<OCIE0);
}

// Setup watch dog timer - TBD
void InitWatchdog(void)
{
}

void Init_SPI_Master(void)
{
    unsigned char inchar;
	
    // Set MOSI and SCK output, all others input
	//  PB5 - MOSI, PB7 - SCK, PB4 - SS\
//	DDRB |= (1<<PB5)|(1<<PB7)|(1<<PB4);
	DDRB |= (1<<PB5)|(1<<PB7);
	PORTB |= 0xA0;  // Set bits MOSI & SCK

	//  Make Flash Chip select PC4 output and set high
	DDRC |= 0x10; 
	PORTC |= 0x10;  
	
	// Enable SPI, Master, set clock rate fck/16 - 500KHz
	//    Select SPI Mode 3 - CPOL=1, CPHA=1
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA)|(1<<SPR0);
	
	// Clear SPI Receive Flag if set
	//if(SPSR&(1<<SPIF)) inchar = SPDR;
	
	// Enable double speed SCK clock - 4MHz
	//  Set SPR1/0 to 0 and SPI2x to 1
	SPCR &= 0xFC;
	SPSR |= (1<<SPI2X);
}

void ChangeBaudRate(unsigned char baudrate)
{
	// Set the baud rate
	UBRRH = 0x00;
	UBRRL = baudrate; 
}

unsigned char CheckEEPROMWritten()
{
 	char P1signature[19];
	unsigned char i;
	
	P1signature[18] = 0;
	for(i=0; i<18; i++)  P1signature[i] = EEPROMread(i);
	if(!strcmp(P1signature,"SOC Robotics V0.87")) return YES;
	else return NO;
}

void Init_EEPROM()
{
	WriteEEPROM(0,"SOC Robotics V0.87");
	EEPROMWrite(19,OSCCAL);   // Write calibration byte
	EEPROMWrite(20,'3');      // Set baud rate to 38400
}

//************************************************************
//  Initialize the AVR - set port conditions for Blackfin
//************************************************************
void Init_AVR(void)
{
    // Setup serial flash constants and variables
	programmode = LFLASH;  // programming serial flash - default
	flashprogrammingactive = NO;
	flashpageloaded = NO;
	flashpageboundarytraversed = NO;
	flashdatainbuffer = NO;
	intelextendedaddress = 0;
	lastaddress = 0;
	showascii = NO;
	oldpage = 0;
	oldbyteoffset = 0;
	xofftransmitted = NO;
	
    // Setup Port A
	InitADC();
	
	// Setup Port B - SPI + General digital IO
	
	//   Setup SPI - SPI default inactive in this release
//    Init_SPI_Master();
	Release_SPI();
	
	// Setup Port C - PF2,0 control + RS-485 + 2-Wire
	//   PC4 - First Serial Flash CS
	DDRC = 0x08;   // PC3 - RS-485 output and PC4 - PF2 Flash CS input
	PORTC = 0x18;  // Set PC4,3 high - deselect Flash CS
	
	//    Turn on real time clock on PC7,6 Timer/Counter 2
	InitRealTimeClock();
	
	// Setup Timer 0 to interrupt a 1msec rate - this interrupt
	// controls four software delay timers used by various 
	// routines
	Init_Timer0();
	
	//    Setup 2-Wire Interface
	//Init_TWI_Master();   // SCL freq = sclk/(16+2*TWBR)*4*TWPS
	
	// Setup Port D  - Blackfin control lines
	//  PD2 - Selects second serial flash
	DDRD = 0xFF;	// output
	PORTD = 0x00;	// all off
	
	// Set initial Blackfin conditions - Boot mode 
	// If BMODE=00 then the Blackfin doesn't control SPI - this may work
	// BMODE1 is PD5
	// BMODE0 is PD4
	BlackfinBootMode(BOOTMODE3);
	// RESETB is PD3 - Release ResetB
	
	// Turn LEDs off
	RED_LED(OFF);
	GREEN_LED(OFF);
	
	// Watchdog timer setup
	InitWatchdog();
	
	// Initialize UART - Interrupt
	// Set the baudrate to 9,600 bps using an 8MHz clock
//	InitUART(BAUDRATE9600); 
//	InitUART(BAUDRATE19200); 
	InitUART(BAUDRATE38400); 
//	InitUART(BAUDRATE57200);  
					
	_SEI(); // Enable interrupts => enable UART interrupts

	//  See if EEPROM setup values exist - if not write them 
	//  otherwise load settings
	if(CheckEEPROMWritten()==NO)  {
	    //puts("no eeprom match\n\r");
	    // Calibrate the internal oscillator
	    CalibrateOsc(QUIET);
		
		// Write default EEPROM P1 AVR information
		Init_EEPROM();
		}
	else {
	    OSCCAL = EEPROMread(19);
		SetNewBaudRate(EEPROMread(20));
		//puts("use eeprom settings\n\r");
		}
}
