/**************************************************************

   Program Title:  P1 AVR Boot/Monitor
   
   Program Name:  P1_AVR_TWI.c	Version: 0.87
   
   Author:  SS	  			    Date:  August 10, 2005
   
   Compiled with ImageCraft ICCAVR Standard V6.29
   
   This file contains routines that implement an TWI Master and Slave
   interrupt driven controller.
   
   Copyright 2004-2005, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright (C) 2005 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Monitor.c,v $
 * Revision 0.55  2005/07/17 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom16v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <stdio.h>
#include "P1_AVR_Uart.h"
#include "P1_AVR_Defines.h"
#include "P1_AVR_Extern.h"

// Motor control Commands
#define    MOTORBOTHFORWARD  	'1'    //    FB - forward both motors
#define    MOTORRIGHTFORWARD  	'2'    //    FR - forward right motor
#define    MOTORLEFTFORWARD  	'3'    //    FL - forward left motor
#define    MOTORBOTHREVERSE  	'4'    //    RB - reverse both motors
#define    MOTORREVERSERIGHT  	'5'    //    RR - reverse right motor
#define    MOTORREVERSELEFT  	'6'    //    RL - reverse left motor
#define    ROTATECW				'c'	   //    Rotate clockwise
#define    ROTATECCW			'w'    //    Rotate counter clockwise
//    Axxx - Acceleration rate xxx - 000 to 999
#define    MOTORACCELERATION  	'7'    
#define    MOTORHALT  	        'h'    //    H - Halt 
#define    LIGHTCONTROL  	    '9'    //    LxH - turn light x[O]N/O[F]F
//    Sxxx - set speed to xxx - 000 to 999 (relative as ratio)
#define    MOTORRIGHTSPEED      'a'
//    Sxxx - set speed to xxx - 000 to 999 (relative as ratio)    
#define    MOTORLEFTSPEED       'b'    
#define    RESETCONTROLLER  	'g'    //    E - reset to defaults

// Initialize the TWI Master interface I2C
//  TBD - pull the default slave address from EEPROM
//        otherwise use default
void  Init_TWI_Master(void)
{
 	  TWBR = 2;  // Set bit rate to about 100K with 8MHz clock
	  TWCR = (1<<TWINT)|(1<<TWEN);  // Turn 2 wire port on  SDA - PC1, SCL - PC0
}

// Initialize the TWI Slave interface I2C
//  Set TWI slave address and optionally General Call
void  Init_TWI_Slave(unsigned char slaveaddress,char generalcall)
{
 	  if(generalcall) TWAR = (slaveaddress<<1) | TWGCE;
	  else TWAR = (slaveaddress<<1);
	  // Turn 2 wire port on, enable slave address recognition,
	  // clear TWINT bit -  SDA - PC1, SCL - PC0
	  TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN);  
}

void TWI_Error(void)
{
}

void TWI_Off(void)
{
 	 TWCR = (1<<TWINT);  // Reset TWI - this allows SDA to go high
}

// Transmit one byte to slave processor via 2-wire interface
unsigned char  TWI_Transmit(unsigned char slaveaddress, unsigned char txdata,
                            unsigned char statusmode)
{
      unsigned char twistatus1,twistatus2,twistatus3;
	  int countervalue;
	  
	  twistatus1=twistatus2=twistatus3 = 0;

	  // Send start command and wait for completion
	  TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTA);
	  while(!(TWCR & (1<<TWINT)))  ;	
	  twistatus1 = TWSR;
	  // Send slave address and write command 
	  TWDR = (slaveaddress<<1) | SLA_W;
	  TWCR = (1<<TWINT)|(1<<TWEN);
	  while(!(TWCR & (1<<TWINT)))  ;  
	  twistatus2 = TWSR; 
	  // Send data byte and wait for transmission to complete
	  TWDR = txdata;    
	  TWCR = (1<<TWINT)|(1<<TWEN);
	  countervalue = 0;
	  while(!(TWCR & (1<<TWINT))) {
//		  if(countervalue++>90) break;
		  } 	 
	  twistatus3 = TWSR;
/*	  // Send second data byte and wait for transmission to complete
	  TWDR = 0x45;    
	  TWCR = (1<<TWINT)|(1<<TWEN);
	  countervalue = 0;
	  while(!(TWCR & (1<<TWINT))) {
		  if(countervalue++>40) break;
		  } 	 
	  twistatus3 = TWSR;
*/
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO); 
	  // Optionally output status bytes
	  if((txdata!=0x55)&(statusmode==VERBOSE)) {
	      puts("\n\rTWI status bytes = "); 
	      bytetohex(twistatus1);
	      puts(hexbyte); putchar(' ');
	      bytetohex(twistatus2);
	      puts(hexbyte); putchar(' '); 
	      bytetohex(twistatus3);
	      puts(hexbyte);
		  }
	  
	  // Send stop command
//	  if(countervalue>90) return 0x02;
//	  else TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO); 

	  return 0x01;  // OK
}

// Receive one byte from slave processor via 2-wire interface
unsigned char TWI_Master_Receive(unsigned char slaveaddress)
{
 	  unsigned char rxdata;
	  
	  rxdata = 0;
 	  TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTA);  // Send start condition
	  while(!(TWCR & (1<<TWINT)))  ;	  // Wait for TWINT flag - start done
	  //if((TWSR & 0xF8) != TWI_START) twi_error();  // See if Start condition
	  TWDR = (slaveaddress<<1) | SLA_R;	 // Request data command
	  TWCR = (1<<TWINT)|(1<<TWEN);		  
	  while(!(TWCR & (1<<TWINT)))  ;	  // Wait for ACK/NACK 
	  if((TWSR & 0xF8) != MT_SLA_ACK) TWI_Error();  // See if ACK/NACK received  

	  TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN);
	  while(!(TWCR & (1<<TWINT)))  ; 	  // Wait for reception to complete
	  rxdata = TWDR;      			  	  // Send data byte 
	  
	  // Get next receive byte
	  /*
	  TWCR = (1<<TWINT)|(1<<TWEN);
	  countervalue = 0;
	  while(!(TWCR & (1<<TWINT))) {
		 if(countervalue++>40) break;
		 } 	
	  inbyte = TWDR;    
	  twistatus4 = TWSR;
	  */
	  
	  //if((TWSR & 0xF8) != MT_DATA_ACK) TWI_Error();  // See if data received
	  TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);   // transmit stop
	  return rxdata;
}
								
void  TestTWI(void)
{
	TWI_Transmit(0x15,0x55,QUIET);
	TWI_Transmit(0x15,0xaa,QUIET);
}

void Sumo1_Pattern(void)
{
    unsigned char pattern[20] = {'1','2','3','5','6','4','c','w',
			 	  			  	 '1','2','3','5','6','4','c','w',
								 '1','4','c','w'};
	char inchar;
	int i;
	
	//puts("\n\rDo repetitive pattern - press any key to stop");
	while(!(inchar=getcharnw())) {
	    for(i=0; i<20; i++) {
	        TWI_Transmit(0x15,pattern[i],QUIET);
		    LongDelay(8);
		    }
		}
	TWI_Transmit(0x15,'h',QUIET);
}

void Sumo1_Controller(void)
{	 
	unsigned char inchar;
	
    //puts("Sumo 1 Command Controller");
	//puts("\n\rEnter command[1,2,3,4,5,6,h,c,w][e to exit]: ");
 	while((inchar=getchar())!='e')  {
	    putchar(inchar);
		if(inchar=='p') Sumo1_Pattern();
		else {
		    if(TWI_Transmit(0x15,inchar,QUIET)!=0x01) {
		        TWI_Off();
		        Init_TWI_Master();
		        }
			}
		//puts("\n\rEnter command[1,2,3,4,5,6,h,c,w][e to exit]: ");
		}
}
void TWI_Write_Slave()
{
    unsigned char inchar,twistatus1,twistatus2,twistatus3;
	int countervalue;
	
    puts(" TWI Master Write Active\n\r");
	Init_TWI_Master();
	puts("Enter character ['e'-exit z-sumo1]: ");
	while((inchar=getchar())!='e')  {
	    countervalue = 0;
		putchar(inchar);
		// Output byte on TWI continuously
		if(inchar=='z') {
		    Sumo1_Controller();
			puts("Press 'e' to exit");
			}
		if(inchar=='r') {
		    while(!(inchar=getcharnw()))  {
			    if(TWI_Transmit(0x15,0x55,VERBOSE)!=0x01) {
		            TWI_Off();
		            Init_TWI_Master();
		            }
			    }
			}
		else {
		    if(TWI_Transmit(0x15,inchar,VERBOSE)!=0x01) {
		        TWI_Off();
		        Init_TWI_Master();
		        }
			}
        puts("\n\rEnter character('e' to exit): ");
	    }
	TWI_Off();
	puts("\n\r");
}

