// *******************************************************
//
//   Program:  P1_Main.c        Version: 0.52 
//
//   Author: Stephen Swift		Date:  Oct 20, 2004
//
//   Main Extern include file for P1 Blackfin Monitor Program.
//
// *******************************************************
/*
 * Copyright (C) 2005-2007 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Main.h,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */

//--------------------------------------------------------------------------//
// External Constants														//
//--------------------------------------------------------------------------//

extern unsigned char programmode,recordtype;
extern unsigned char ip[4],ipaddress[4],ipmask[4],ipgateway[4],ethernetmac[6];
extern unsigned int bytesonline,inteladdress,extendedaddress;
extern unsigned char crc,intelbinary[40];  // length can vary
extern char hexbyte[3],hexaddress[5],SPI_State,endofintelhex,intelhexecho,intelhexprintmode;
extern char daytab[2][13];
extern unsigned char FlashAddress[3];
extern int second,minute,hour,day,year;
extern int f1pagesize,f2pagesize,fillindex,loopcnt;
extern long f1bytesize,f2bytesize;
extern unsigned long addressptr;

//  Flash initialization and control paramters
extern int currentflashbyteoffset;  // Current byte in flash RAM buffer
extern int currentflashpage;   // Current flash page being loaded
extern int currentflashchip;   // Current flashchip being accessed - 1 or 2
extern char flashbufferloaded;  // Status of flash buffer load - YES/NO
extern char flashpageprogrammed; // Status of flash page programming - YES/NO
extern char flashbufferempty;   // True if data loaded into flash RAM buffer but not programmed


/* Static Variables */
extern unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
extern unsigned char UART_RxHead;
extern unsigned char UART_RxTail;
extern unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
extern unsigned char UART_TxHead;
extern unsigned char UART_TxTail;
extern char  xoffsent, xonsent, enablexonoff, echoinchars, sendxonxoff;
extern char  rxcnt, checkflag;

//--------------------------------------------------------------------------//
// Prototypes																//
//--------------------------------------------------------------------------//

//  P1_Init File Functions
extern void Init_Flags(void);
extern void Init_Timers(void);
extern void Init_EBIU(void);
extern void Init_Interrupts(void);
extern void Init_DMA_CH0(unsigned char);
extern void Init_DMA_CH0_CM200(void);
extern void Init_MasterSPI(void);
extern void Init_PLL(void);
extern void Init_PPI_CM64(unsigned char);
extern void Init_PPI_CM200(void);
extern void Init_SDRAM(void);
extern void Init_UART(long);

// P1_Video File Functions
extern void YUVtoRGB(short,short);
extern void RGBtoYUV(void);
extern void CreateColor(short,short,long,char);
extern void CreateColorW(short,short,long,short);
extern void CreateBox(short,short,long);

// In UART read/write
extern void putchar(unsigned char);
extern unsigned char getchar(void);
extern unsigned char getcharnw(void);
extern void puts(char *);
extern void printf(char *fmt, ...);
extern void scanf(char *fmt, ...);
extern void printint(int,char,char);
extern void PrintHexByte(unsigned char);
extern void PrintHexWord(unsigned int);
extern void printlonghex(unsigned long);

//  LDRReader File Functions
extern unsigned char ReadSDRAMByte(unsigned long);
extern void BinaryFlag(unsigned short);
extern unsigned long getlong(void);
extern unsigned int getword(void);
extern void LoadElfImage(unsigned long);

// P1_I2C File Functions
extern void Set_SDA(char);
extern void Set_SCL(char);
extern void Init_I2C(void);
extern void Pulse_SCL(void);
extern void I2C_Start(void);
extern void I2C_Stop(void);
extern void I2C_RepeatedStart(void);
extern int Write_I2C_Control(unsigned char);
extern int Write_I2C_Byte(unsigned char);
extern unsigned char Read_I2C_Byte(void);
extern int Write_CM64(unsigned char,unsigned char);
extern unsigned char Read_CM64(unsigned char);
extern void Init_SAA7113(short);
extern void Reset_SAA7113(void);
extern short  CM64_VideoStatus(void);
extern void Halt_VideoDMAPPI(void);
extern void Init_CM64(short,unsigned char);
extern void CM200_Power(unsigned char);
extern void Reset_CM200(void);
extern int Write_CM200(unsigned char,unsigned char);
extern unsigned char Read_CM200(unsigned char);
extern void Init_ICM109S(void);
extern void Init_CM200(void);

//  P1_SPI_Flash File Functions
extern void FlashChipSelect(int,int);
extern void SPI_MasterTransmit(unsigned char);
extern unsigned char SPI_MasterReceive(void);
extern void ConvertFlashAddress(long);
extern void SPI_StartReadFlashPage(long);
extern void SPI_ReadFlashBuffer(long,int);
extern void DisplayExtendedAddress(unsigned int);
extern void DisplaySerialFlash(long,long,char);
extern unsigned char FlashStatus(int);
extern void FlashPageToBuffer(int,int,int);
extern void FlashPageErase(int,int);
extern void FlashPageWrite(int,int,int);
extern void FlashBytesToBuffer(int);
extern void WriteFlashByte(int,unsigned int,unsigned char);
extern void ProgramFlashPage(int,unsigned int,int,int);
extern void ResetFlashVars(void);
extern void ProgramFlash(void);
extern void Init_Flash(void);
extern void FillFlashMemory(long,long,unsigned char,unsigned char);
extern void CheckFlash(void);

// P1_Utilities File Functions
extern void Delay(long);
extern void Delaynano(long);
extern unsigned char hextobyte(unsigned char *instrg);
extern unsigned long getlonghex(void);
extern int getint(void);
extern char *getstrg(void);
extern void printf(char *fm, ...);
extern void scanf(char *fmt, ...);
extern void putchar(unsigned char);
extern unsigned char getcharnw(void);
extern unsigned char getchar(void); 
extern void puts(char *instrg);
extern void printint(int,char,char);
extern void bytetohex(unsigned char);
extern void addresstohex(unsigned int);
extern void PrintHexByte(unsigned char);
extern void PrintHexWord(unsigned int);
extern void Init_RealTimeClock(void);
extern void Get_DateTime(void);
extern void Set_DateTime(void);
extern void month_day(int,int, int *pmonth, int *pday);
extern int day_of_year(int, int, int);
extern char *month_name(int);
extern void printdate(void);
extern void printdatetime(void);
extern char echochar(void);
extern void Enter_DateTime(void);
extern void InitWatchdog(void);
extern void WriteBlackfinMemory(long,unsigned char);
extern unsigned char ReadBlackfinMemory(long);
extern void printlonghex(unsigned long);
extern void DisplayIntelRecord(unsigned int,int,unsigned char *);
extern void DisplayBlackfinSDRAMMemory(long,long);
extern void PrintMemory(void);
extern unsigned char gethexbyte(void);
extern int checkcrc(void);
extern unsigned int getinteladdress(void);
extern int DecodeIntelHex(void);
extern void ProgramSDRAM(void);
extern void ResetFlashVars(void);
extern void ProgramFlash(void);
extern void ProgramMemory(void);
extern unsigned char gethextobyte(void);
extern void FillSDRAMMemory(long,long,unsigned char);
extern void FillMemoryCommand(void);
extern void LoadMemory(void);
extern void MoveMemory(char,char,long,long,long);
extern void MoveMemoryCommand(void);
extern void ProgramVersion(void);
extern void CountPeriodStart(void);
extern long CountPeriod(void);
extern long CountUPeriod(void);
extern void TestBlackfin(void);
extern void BootApplication(void);
extern void ExtendedMessage(void);
extern void SignOnMessage(void);
extern void DecodeIP(void);
extern void EncodeIP(char *instrg);
extern void EncodeMAC(char *instrg);
extern void DecodeMAC(void);
extern void Init_Internet(void);
extern void InternetSettings(void);
extern void GetInternetSettings(void);
extern void CompareMemory(void);
extern void Preferences(void);
extern int SDRAMSize(void);
extern void Specifications(void);
extern int  Init_AVR(void);
extern void Init_Blackfin(void);
extern void GCodeTest(void);
extern void YUVtoGREY(short,short);

extern void SpartanTest(void);

