/*****************************************************************************
**																			**
**	 Name: 	P1 Pirana Boot Monitor										    **	
**																			**
******************************************************************************

Project Name:	P1 Blackfin Monitor.c

Date Modified:	20/10/04		SS		Ver 0.65

Software:		VisualDSP++ 3.5

Hardware:		P1 Pirana Embedded Video Processor

Purpose:		Implements a basic Monitor for the P1 Pirana Embedded video 
				processor Code is provided to program the CM64 Video Adapter, 
				Serial port, and APS12 10/100 Ethernet Chip.
				
				TODO - add rudimentary OS for eBoot support
				       and/or uCLinux boot support

*/
/*
 * Copyright (C) 2005 by SOC Machines, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Blackfin_Monitor.c,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
#include "P1_Main.h"
#include "ccblkfn.h"
#include "sysreg.h"
#include "P1_I2C.h"
#include "SAA7113.h"  


//  Variable declarations
unsigned char programmode,recordtype;
unsigned char ip[4],ipaddress[4],ipmask[4],ipgateway[4],ethernetmac[6];
unsigned int bytesonline,inteladdress,extendedaddress;
unsigned char crc,intelbinary[40];  // length can vary
char hexbyte[3],hexaddress[5],SPI_State,endofintelhex,intelhexecho,intelhexprintmode;
int second,minute,hour,day,year;

//--------------------------------------------------------------------------//
// Variables																//
//--------------------------------------------------------------------------//

/* Static Variables */
extern unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
extern unsigned char UART_RxHead;
extern unsigned char UART_RxTail;
extern unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
extern unsigned char UART_TxHead;
extern unsigned char UART_TxTail;
extern char  xoffsent, xonsent, enablexonoff, echoinchars, sendxonxoff;
extern char  rxcnt, checkflag;

int f1pagesize,f2pagesize,fillindex;
long f1bytesize,f2bytesize;


//****************************************************************
//****************************************************************

//  Delay for the selected number of microseconds
//    A 130.5MHz SCLK counts 118.6 ticks per microsecond
//    Max delay is 36.2 seconds
void Delay(long microseconds)
{
	long countperiod;
	
	countperiod = microseconds * 131;
	//  Use timer 1 as count up timer
	*pTIMER1_CONFIG	= 0x0049;
	*pTIMER1_PERIOD	= countperiod;
	*pTIMER1_WIDTH = 0x00000006;
	*pTIMER_ENABLE |= 0x0002;
	// Timer stops once period count is reached
	*pTIMER_DISABLE |= 0x0002;
	
	while(*pTIMER1_COUNTER < countperiod) ;
	*pTIMER_DISABLE &= 0x000D;	
	*pTIMER_ENABLE &= 0x000D; 
}

//  Delay for the selected number of nanoseconds
//    A 118.8MHz SCLK counts 118.6 ticks per microsecond
//    or Max delay is 36.2 seconds
void Delaynano(long nanoseconds)
{
	long countperiod;
	
	countperiod = nanoseconds*8;
	//  Use timer 1 as count up timer
	*pTIMER1_CONFIG	= 0x0049;
	*pTIMER1_PERIOD	= countperiod;
	*pTIMER1_WIDTH = 0x00000006;
	*pTIMER_ENABLE |= 0x0002;
	// Timer stops once period count is reached
	*pTIMER_DISABLE |= 0x0002;
	
	while(*pTIMER1_COUNTER < countperiod) ;
	*pTIMER_DISABLE &= 0x000D;	
	*pTIMER_ENABLE &= 0x000D; 
}

void FlashChipSelect(int FlashChip,int mode)
{
	Delay(1);
	if((FlashChip==1)&&(mode==LOW)) *pSPI_FLG = 0xFB04;
	else if((FlashChip==1)&&(mode==HIGH)) *pSPI_FLG = 0xFF04;
	else if(FlashChip==2&&mode==LOW) *pSPI_FLG &= 0xF7FF;
	else if(FlashChip==2&&mode==HIGH) *pSPI_FLG |= 0x0800;
	Delay(1);
}

void putchar(unsigned char data)
{
	unsigned char tmphead;
	
	// Do polled transmit if 1 else interrupt version
	if(0) {
		while(!(*pUART_LSR&0x0040)) ;
		*pUART_THR = data;
		}
	else {
		
	/* Calculate buffer index */
	tmphead = (UART_TxHead + 1) & UART_TX_BUFFER_MASK; 
		
	/* Wait for free space in buffer */
	while (tmphead == UART_TxTail) ;
	
	UART_TxBuf[tmphead] = data; // store data in buffer 
	UART_TxHead = tmphead;      // store new index
	
//	*pUART_THR = UART_TxBuf[tmphead];
	
	//  Fire TX interrupt
	*pUART_IER |= ETBEI;
//	*pSIC_IMASK = 0x0000C000;
	}
}

int loopcnt;

//  Read uart - no wait
unsigned char getcharnw(void)
{
	unsigned char tmptail,inchar;
	char rxtmp;
	

	// Do polled receive if 1 else interrupt version
	if(0) {
		if(*pUART_LSR&0x0001) {
			inchar = *pUART_RBR;
			return inchar;
			}
		else return 0; 
		}
	else { 
/*	while ( UART_RxHead == UART_RxTail ) ; */ /* wait for incomming data */
	if(UART_RxHead==UART_RxTail) return 0; /* nothing to get */
		
	tmptail = UART_RxTail + 1; /* calculate buffer index */
	if(tmptail>=UART_RX_BUFFER_SIZE) tmptail = 0;
	UART_RxTail = tmptail; /* store new index */
	
 
//	if(loopcnt++>100) {
//		putchar('+');
//		loopcnt = 0;
//		}

	asm("cli r0;");
	rxcnt--;
	rxtmp = rxcnt;
	asm("sti r0;");

	if(checkflag == YES)  {
		//putchar('@');	
		checkflag = NO; 
		}
//	if(rxtmp < 0) putchar('<');
//	if(rxtmp > 0x3f) putchar('>');
	
	//  Allow tx at remote serial device again - buffer has more space
	if((rxtmp<(UART_RX_BUFFER_SIZE-50))&&(xoffsent==YES)&&(enablexonoff==YES)) {
		if(xonsent==NO);  {
			putchar(XON); putchar('$');
			xonsent = YES;
			}
//		xoffsent = NO;
		}
	return UART_RxBuf[tmptail]; /* return data */
	}
}


// Getchar waits until a valid character is received
unsigned char getchar() 
{
	unsigned char inchar;
	
 	while(!(inchar=getcharnw())) ;
 	return inchar;
}


void puts(char *instrg)
{
	int i;
	
	i = 0;
	while(instrg[i++] != 0x00)  putchar(instrg[i-1]);
}

//  Print integer - charcnt is number of characters to display, leadchartype 
//   is either '0' or ' '.  If leadchartype is 0x00 suppress leading characters
void printint(int intvalue,char charcnt,char leadchartype)
{
 	 char outvalue[6],nolead;
 	 int  i,tenthousands,thousands,hundreds,tens;
	 
	 if(intvalue < 0) {
	 	  putchar('-');
	 	  intvalue *= -1;
		  }
	 tenthousands = intvalue/10000;
	 intvalue -= (tenthousands*10000);
	 thousands = intvalue/1000;
	 intvalue -= (thousands*1000);
	 hundreds = intvalue/100;
	 intvalue -= (hundreds*100);
	 tens = intvalue/10;
	 nolead = 0x00;
	 // TODO - suppress only leading zero's
	 if(tenthousands)  {outvalue[0] = '0' + tenthousands; nolead=0x01;}
	 else outvalue[0] = leadchartype;
	 if(thousands|nolead) {outvalue[1] = '0' + thousands;nolead=0x01;}
	 else outvalue[1] = leadchartype;
	 if(hundreds|nolead) {outvalue[2] = '0' + hundreds;nolead=0x01;}
	 else outvalue[2] = leadchartype;
	 if(tens|nolead) outvalue[3] = '0' + tens;
	 else outvalue[3] = leadchartype;
	 outvalue[4] = '0' + intvalue%10; 
	 outvalue[5] = 0x00;
	 for(i=0; i<charcnt; i++) {
	 	if(outvalue[(5-charcnt)+i]) putchar(outvalue[(5-charcnt)+i]);
	 	}
}

void bytetohex(unsigned char byte)
{
    unsigned char inbyte,inchar;
	
	inbyte = byte & 0x0f;
	if(inbyte > 9) hexbyte[1] = (inbyte - 9) + 0x40;
	else hexbyte[1] = 0x30 + inbyte;
	inbyte = byte & 0xf0;
	inbyte >>= 4;
	if(inbyte >9) hexbyte[0] = (inbyte - 9) + 0x40;
	else hexbyte[0] = 0x30 + inbyte;
    hexbyte[2] = 0x00;
}

void addresstohex(unsigned int address)
{
 	 unsigned char lowbyte,highbyte;
	 int intbyte;
	 
	 lowbyte = address & 0x00FF;
	 bytetohex(lowbyte);
	 hexaddress[2] = hexbyte[0];
	 hexaddress[3] = hexbyte[1];
	 intbyte = address;
	 highbyte = address/256;
	 bytetohex(highbyte);
	 hexaddress[0] = hexbyte[0];
	 hexaddress[1] = hexbyte[1];
	 hexaddress[4] = 0x00;
}

void PrintHexByte(unsigned char inchar)
{
	bytetohex(inchar);
	puts(hexbyte);
}

void PrintHexWord(unsigned int invalue)
{
	addresstohex(invalue);
	puts(hexaddress);
}

// Initialize real time clock
void Init_RealTimeClock()
{
	*pRTC_PREN = 0x0001;
	year = 2004;
	
	// TODO - set clock to default value
	*pRTC_STAT = 0x00020000;
}

// Get Date and time from serial port
void Get_DateTime()
{
	long datetime;
	
	datetime = 0;
	datetime = *pRTC_STAT;
	second = datetime & 0x0000003F;
	minute = (datetime >> 6) & 0x0000003F;
	hour = (datetime >> 12) & 0x0000001F;
	day = (datetime >> 17) & 0x00007FFF;
}

// Set current date and time 
//  Format is 10/9/04 13:10:30
void Set_DateTime()
{
	long newdatetime;
	
	newdatetime = day*131072 + hour*4096 + minute*64 + second;
	*pRTC_STAT = newdatetime;
}

char daytab[2][13] = {
	{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
	{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
	};

//  Month_day: set month,day from day of year
void month_day(int yr,int yearday, int *pmonth, int *pday)
{
	int i, leap;
	
	leap = yr%4 == 0 && yr%100 != 0 || yr%400 == 0;
	for(i=1; yearday > daytab[leap][i]; i++)
		yearday -= daytab[leap][i];
	*pmonth = i;
	*pday = yearday;
}

int day_of_year(int yr, int pmonth, int dayofmonth)
{
	int i,leap;
	
	leap = yr%4 == 0 && yr%100 != 0 || yr%400 == 0;
	for(i=1; i < pmonth; i++)
		dayofmonth += daytab[leap][i];
	return dayofmonth;
}

// Return month as ASCII string
char *month_name(int n)
{
	static char *name[]={
		" ",
		"January","February","March",
		"April","May","June",
		"July","August","September",
		"October","November","December" };
	
	return (n<1 || n>12) ? name[0] : name[n];
}

void printdate()
{
	int d,m;
	
	month_day(year,day,&m,&d);
	puts(month_name(m)); putchar(' ');
	printint(d,2,0);     puts(", ");
	printint(year,4,0);  putchar(' ');	
}

void printdatetime()
{
	printdate();  putchar(' ');
	printint(hour,2,'0'); putchar(':');
	printint(minute,2,'0'); putchar(':');
	printint(second,2,'0');

}

char echochar()
{
	char inchar;
	
	inchar = getchar();
	putchar(inchar);
	return inchar;
}

void Enter_DateTime()
{
	char inchar,strg[3];
	int month,yearstub,dayofmonth;
	
	strg[2] = 0;
	if((inchar=getchar())!=CR) {
		// get month
		strg[0] = inchar; putchar(inchar);
		strg[1] = echochar();
		month = atoi(strg);
		echochar();  // read '/'
		// get day
		strg[0] = echochar();
		strg[1] = echochar();
		dayofmonth = atoi(strg);
		day = day_of_year(year,month,dayofmonth);
		echochar();  // read '/'
		// get year
		strg[0] = echochar();
		strg[1] = echochar();
		yearstub = atoi(strg);
		if(yearstub>30) year = yearstub + 1900;
		else year = yearstub + 2000;
		echochar();  // read ' '
		// get hour
		strg[0] = echochar();
		strg[1] = echochar();
		hour = atoi(strg);
		echochar();  // read ':'
		// get minute
		strg[0] = echochar();
		strg[1] = echochar();
		minute = atoi(strg);
		echochar();  // read ':'
		// get second
		strg[0] = echochar();
		strg[1] = echochar();
		second = atoi(strg);
		
		// wait until carriage return to exit
		while((inchar=getchar())!=CR) putchar(inchar);
		Set_DateTime();		
		}
}

// Initialize watchdog timer
void InitWatchdog()
{
	
}

// Convert ASCII hex to binary
unsigned char hextobyte(unsigned char *instrg)
{
 	 unsigned char lownibble,highnibble;
	 
 	 //  Convert high byte
	 if((instrg[0] >= 'A')&(instrg[0]<='F')) highnibble = instrg[0] - 'A' + 10;
	 else if((instrg[0] >= 'a')&(instrg[0]<='f')) highnibble = instrg[0] - 'a' + 10;
	 else highnibble = instrg[0] - '0';

	 //  Convert low byte
	 if((instrg[1] >= 'A')&(instrg[1]<='F')) lownibble = instrg[1] - 'A' + 10;
	 else if((instrg[1] >= 'a')&(instrg[1]<='f')) lownibble = instrg[1] - 'a' + 10;
	 else lownibble = instrg[1] - '0';
	 
	 highnibble = highnibble<<4;
	 return highnibble + lownibble;
}

void WriteBlackfinMemory(int address,unsigned char byte)
{
	*(pSDRAMBot+address+extendedaddress) = byte;
}

unsigned char ReadBlackfinMemory(int address)
{
 	unsigned char flashbyte,rbyte;
	
	rbyte = *(pSDRAMBot+address+extendedaddress*65536);
	return (rbyte);                              
}

void DisplayExtendedAddress()
{
	unsigned char crcvalue,tempvalue;
	
	crcvalue = 0x00;
	puts(":02000004");
	crcvalue += 0x02;
	crcvalue += 0x04;
	addresstohex(extendedaddress);
	puts(hexaddress);
	tempvalue = (extendedaddress&0xff00)>>8;
	crcvalue += tempvalue;
	tempvalue = extendedaddress&0x00ff;
	crcvalue += tempvalue;
	bytetohex((~crcvalue)+0x01);
	puts(hexbyte);
	puts("\n\r");
}

// TODO - break if no reception
void SPI_MasterTransmit(unsigned char cData)
{
	unsigned int inword,outword;
    long cnt;
	
    //  Enable transmit operation
    *pSPI_CTL = 0x5C0D;
	cnt = 0;

	// Wait until TX buffer empty - full=1
	if(SPI_State==0x01) 
	    while(*pSPI_STAT & 0x0004) ;
	

    // Start transmisstion
	*pSPI_TDBR = cData;
		
	// Do dummy read to transfer data
//	inword = *pSPI_RDBR;
	
	//  See if error bits are set - clear if set
	inword = *pSPI_STAT;
	if(inword&0x0014) *pSPI_STAT |= 0x0014;
	
	// Wait for transmission to complete
//	puts("wait for spi tx to complete\n\r");
	while(!(*pSPI_STAT & 0x0001)) ;
	
	SPI_State = 0x01;
}

// TODO - break if no reception
unsigned char SPI_MasterReceive(void)
{
	unsigned int inword;
    // Wait for reception to complete
    // Is a dummy read required?
    *pSPI_CTL = 0x5C0C;
    
//	*pSPI_TDBR = 0x00;
	SPI_State = 0x02;

	inword = *pSPI_RDBR;
	
	while(!(*pSPI_STAT & 0x0020)) ;
	
	return (*pSPI_RDBR & 0x00ff);
}

unsigned char FlashAddress[3];

//  Read address consists of 4 zero bits+11 page bits and 9 byte start
//    AT45DB041 has 264 bytes per page - 2048 pages
//    AT45DB081 has 264 bytes per page - 
//    AT45DB161 has 512 bytes per page
//    264 does not divide into 2^16 evenly
void ConvertFlashAddress(int address)
{
 	int pages,bytestart,superpages;
 	long superaddress;
	
 	superaddress = extendedaddress*65536+address;
 	if(superaddress < f1bytesize) {
	    pages = superaddress/264;
	    superpages = pages/128;
	    bytestart = superaddress%264;
        FlashAddress[0] = superpages&0x00FF;
	    FlashAddress[1] = ((pages&0x00ff)<<1) + ((bytestart>>8)&0x00FF);
	    FlashAddress[2] = bytestart & 0x00FF;
 		}
 	else {
 		FlashAddress[0] = FlashAddress[1] = FlashAddress[2] = 0;
 		}
}

//  Flash continuous read command for AT45DB041B
//  The AT45DB041B has 2048 264byte pages
void SPI_StartReadFlashPage(int address)
{
 	 int i;
	 
	 // Send Continuous read command
 	 SPI_MasterTransmit(0x68);
//	 puts("Send read command\n\r");
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
//	 puts("Address\n\r");
	 
	 //  32 dummy bits
	 for(i=0; i<4; i++)  SPI_MasterTransmit(0x00);
//	 puts("Send dummy bits\n\r");
}

//  Flash Buffer Read command for AT45DB041B
void SPI_ReadFlashBuffer(int address,int buffernumber)
{
 	 int i;
	 
	 // Send buffer read command
	 if(buffernumber==1) SPI_MasterTransmit(0x54);
	 else SPI_MasterTransmit(0x56);

//	 puts("Send read command\n\r");
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
//	 puts("Address\n\r");
	 
	 //  8 dummy bits
	 SPI_MasterTransmit(0x00);
//	 puts("Send dummy bits\n\r");
}

void printlonghex(unsigned long address)
{
	unsigned char inbyte;
	
	puts("0x");
	inbyte = address/16777216;
	bytetohex(inbyte);  puts(hexbyte);
	inbyte = (address/65536)&0x000000FF;
	bytetohex(inbyte);  puts(hexbyte);
	inbyte = (address/256)&0x000000FF;
	bytetohex(inbyte);  puts(hexbyte);
	inbyte = address&0x000000FF;
	bytetohex(inbyte);  puts(hexbyte);
}

// Read large hex value (32 bits) - skip leading space characters
//   Ignore leading "0x" and terminate on CR, LF, ',' or ' ' characters
unsigned long getlonghex()
{
	unsigned char inbyte;
	char instrg[12],inchar;
	unsigned char strg[3];
	int i,cnt,multiplier,skip;
	unsigned long address;
	
	skip = 0;
	// skip leading space characters
	while((inchar=getchar())==' ')  putchar(inchar);
	instrg[0] = inchar;
	instrg[1] = 0x00;
	putchar(inchar);

	//  Read characters until termination character found
	for(i=1; i<11; i++)  {
		inchar = getchar();
		if((inchar==' ')||(inchar==CR)||(inchar==',')||(inchar==LF)) {
			instrg[i] = 0x00;
			break;
			}
		instrg[i] = inchar;
		putchar(inchar);
		}
	putchar(inchar);
		
	//  Strip leading "0x" if present
	if((instrg[0]=='0')&&((instrg[1]=='x')||(instrg[1]=='X')))
	   for(i=2; i<12; i++)  instrg[i-2] = instrg[i];
	// Convert hex starting at end of string

	cnt = 0;
	for(i=0; i<12; i++)  {
		if(instrg[i] == 0) break;
		cnt++;
		}
	// Convert hex pairs (checking for isolated character)
	strg[2] = 0;
	address = 0;
	multiplier = 1;
	while(cnt>0) {
		strg[1] = instrg[cnt-1];
		cnt--;
		if(cnt>=0) { strg[0] = instrg[cnt-1]; cnt--; }
		else {strg[0] = '0'; cnt--; }
		inbyte = hextobyte(strg);
		if(multiplier==1) address += inbyte;
		else if(multiplier==2) address += inbyte*256;
		else if(multiplier==3) address += inbyte*65536;
		else if(multiplier==4) address += inbyte*16777216;
		multiplier++;
		}
	
	return address;
}

//  Display P1 Blackfin Flash memory
//
//    There are two Serial Flash devices selected by PF2 and PF3
//
//    If extendedaddress nonzero or numbyte > 65536 then output 
//    Extended Address Record at the appropriate time
//
//   TODO - fix up extended addressing
//
//   Note:  The Atmel Flash must be read continuously or incorrect data is 
//          returned.
void PrintFlash(unsigned int address,long numbytes,char readmode)
{
 	 unsigned char i,byte,bytesonlastline,ascii[20],asciicnt,crcvalue,tempvalue;
	 long linecnt,numlines;

	 //  Set PF2 low - Flash CS
//	 puts("CS Low\n\r");
	 FlashChipSelect(1,LOW);
	 
//	 if(readmode==READCONTINUOUS)  SPI_StartReadFlashPage(address);
//	 else if(readmode==READBUFFER) SPI_ReadFlashBuffer(address,1);
//	 puts("Send Continuous Read command\n\r");

//  Output extended address record if extendedaddress > 0
     //    
     //   TODO - take care of numbytes > 65536
     if(extendedaddress) DisplayExtendedAddress();

     linecnt = asciicnt = 0;
	 bytesonline = INTELHEXBYTESPERLINE;
	 
	 // debug
//	 bytesonline = 20;	 
	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
//	 puts("numlines="); printint(numlines,5,0); puts("\n\r");
//	 puts("bytesonlastline="); printint(bytesonlastline,4,0); puts("\n\r");
	 while(linecnt<numlines) {
	     putchar(':');
	     crcvalue = 0;
		 //  Output bytes on line
		 bytetohex(bytesonline);
		 puts(hexbyte);
	     byte = bytesonline;
		 crcvalue += byte;
	     //  Output hex start address
	     addresstohex(address+linecnt*bytesonline);
	     puts(hexaddress);
		 tempvalue = ((address+linecnt*bytesonline)&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = (address+linecnt*bytesonline)&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
	 	 //  Set PF2 low - Flash CS
		 //	 puts("CS Low\n\r");
	 	 FlashChipSelect(1,LOW);
		 if(readmode==READCONTINUOUS)  SPI_StartReadFlashPage(address+linecnt*bytesonline);
		 else if(readmode==READBUFFER) SPI_ReadFlashBuffer(address+linecnt*bytesonline,1);
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = SPI_MasterReceive();
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         }
	         
	     // Set PF2 high - deselect Flash chip
		 FlashChipSelect(1,HIGH);
	     for(i=0; i<bytesonline; i++) {
	         bytetohex(ascii[i]);
	 	     puts(hexbyte);
	     	 }
		 linecnt++;
		 asciicnt = 0;
		 
		 // Output CRC
		 bytetohex((~crcvalue)+0x01);
		 puts(hexbyte);
		 if(intelhexprintmode==LONGMODE) {
		 	puts("  ");
		 	for(i=0; i<bytesonline; i++) {
		     	if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 	else putchar('.');
			 	}
		 	}
		 if(intelhexprintmode==LONGMODE)  puts("\n\r");  
		 else puts("\n");
		 }
	 if(bytesonlastline) {
	 	 crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonlastline);
		 puts(hexbyte);
	     byte = bytesonlastline;
		 crcvalue += byte;
	     //  Output hex start address
	     addresstohex(address+linecnt*bytesonline);
	     puts(hexaddress);
		 tempvalue = ((address+linecnt*bytesonline)&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = (address+linecnt*bytesonline)&0x00ff;
		 crcvalue += tempvalue;
	 	 //  Set PF2 low - Flash CS
		 //	 puts("CS Low\n\r");
	 	 FlashChipSelect(1,LOW);
		 if(readmode==READCONTINUOUS)  SPI_StartReadFlashPage(address+linecnt*bytesonline);
		 else if(readmode==READBUFFER) SPI_ReadFlashBuffer(address+linecnt*bytesonline,1);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = SPI_MasterReceive();
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         }
		 // Set PF2 high - deselect Flash chip
		 FlashChipSelect(1,HIGH);
	     for(i=0; i<bytesonlastline; i++) {
	         bytetohex(ascii[i]);
	 	     puts(hexbyte);
	         }
		 // Output CRC
		 bytetohex((~crcvalue)+0x01);
		 puts(hexbyte);
		 if(intelhexprintmode==LONGMODE) {
		 	puts("  ");
		 	for(i=0; i<bytesonlastline; i++) {
		     	if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 	else putchar('.');
			 	}
		 	}
		 if(intelhexprintmode==LONGMODE)  puts("\n\r");  
		 else puts("\n");
		 }
		 
	
    // Output End of File Record
	puts(":00000001FF\n\r");
}

void PrintIntelHexRecord(unsigned int address,int numbytes)
{ 	 
	unsigned char i,byte,ascii[16],asciicnt,crcvalue,tempvalue;
	unsigned int newaddress;

	crcvalue = asciicnt = 0;
	putchar(':');
	
	//  Output bytes on line
	byte = numbytes;
	bytetohex(byte);
	crcvalue += byte;
	puts(hexbyte);
	
	//  Output hex start address
	addresstohex(address);
	tempvalue = (address&0xff00)>>8;
	crcvalue += tempvalue;
	tempvalue = address&0x00ff;
	crcvalue += tempvalue;
	puts(hexaddress);
	
	//  Output 00
	puts("00");
	
	//  Output data
	for(i=0; i<numbytes; i++) {
	    byte = ReadBlackfinMemory(address+i);
	    ascii[asciicnt++] = byte;
		crcvalue += byte;
	    bytetohex(byte);
	 	puts(hexbyte);
	    }
	asciicnt = 0;
	
	// Output CRC
	bytetohex((~crcvalue)+0x01);
	puts(hexbyte);
	if(intelhexprintmode==LONGMODE) {
	   puts("  ");
	   for(i=0; i<numbytes; i++) {
	       if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
	       else putchar('.');
	       }
	   }
	if(intelhexprintmode==LONGMODE)  puts("\n\r");  
	else puts("\n");
}

//  Display P1 Blackfin  memory
//
//    If extendedaddress nonzero or numbyte > 65536 then output 
//    Extended Address Record at the appropriate time
//    Output 16 bytes per line
void PrintBlackfinMemory(unsigned int address,unsigned long numbytes)
{
 	 unsigned char i,byte,ascii[16],asciicnt,crcvalue,tempvalue,finished;
	 long linecnt,numlines,bankcnt,numbanks;
	 unsigned int newaddress,bytesinbank;
	 int bytesonline,bytesonlastline;

	 if(extendedaddress)  DisplayExtendedAddress();
	 
	 bankcnt = 0;
	 finished = 1;
	 bytesonline = INTELHEXBYTESPERLINE;
	 numbanks = ((numbytes+address-1)/65536) + 1;
//	 puts("numbanks="); printint(numbanks,4,0); puts("\n\r");
	 if((65536-address)>=numbytes)  bytesinbank = numbytes;
	 else bytesinbank = 65536-address;
//	 puts("bytesinbank="); printint(bytesinbank,4,0); puts("\n\r");
	 numbytes -= bytesinbank;
	 while((bankcnt<numbanks)&&finished) {
         //  Output extended address record if extendedaddress > 0
         linecnt = 0;
         if(bankcnt>0) { 
         	DisplayExtendedAddress(); 
//        	puts("bankcnt="); printint(bankcnt,4,0); puts("\n\r");
         	}

	     numlines = bytesinbank/bytesonline;
	     bytesonlastline = bytesinbank - numlines*bytesonline;
	     while(linecnt<numlines) {
	     	 PrintIntelHexRecord(address+linecnt*16,bytesonline);
	     	 linecnt++;
	     	 }
	     if(bytesonlastline) 
	     	 PrintIntelHexRecord(address+linecnt*16,bytesonlastline);
		 bankcnt++;
		 if(numbytes>=65536) bytesinbank = 65536;
		 else bytesinbank = numbytes;
//	     puts(" bytesinbank="); printint(bytesinbank,5,0);
	     numbytes -= bytesinbank;
	     extendedaddress++;
//	     if(getcharnw()) finished = 0;
	     }
	 // Output End of File Record
	 puts(":00000001FF\n\r");
}

//  Display Blackfin processor SDRAM or Flash memory
void PrintMemory()
{
    unsigned char inchar;
    unsigned int address;
    long longaddress,length;
	
	inchar=getchar();
	putchar(inchar);
	switch (inchar) {
		case 's': case 'S':
		    puts("\n\rDisplay Blackfin memory from (address, length): ");
		    longaddress = getlonghex();
		    extendedaddress = longaddress/65536;
		    address = longaddress - extendedaddress*65536;
		    length = getlonghex();
		    puts("\n\r");
		    PrintBlackfinMemory(address,length);
		    extendedaddress = 0;
		    break;
		case 'f': case 'F':
		    puts("\n\rDisplay Flash memory from (address, length): ");
		    longaddress = getlonghex();
		    extendedaddress = longaddress/65536;
		    address = longaddress - extendedaddress*65536;
		    length = getlonghex();
		    puts("\n\r");
		    PrintFlash(address,length,READCONTINUOUS);
//		    PrintFlash(address,length,READBUFFER);
		    extendedaddress = 0;
		    break;
		default:
		    puts("\n\rUnsupported command\n\r");
		    break;
	    }
}

//  Read 2 ASCII Hex characters and return as byte
unsigned char gethexbyte()
{
	unsigned char strg[3],inbyte;
	
	strg[0] = getchar(); if(intelhexecho) putchar(strg[0]);
	strg[1] = getchar(); if(intelhexecho) putchar(strg[1]);
	strg[2] = 0;
	inbyte = hextobyte(strg);
	crc += inbyte;
	return inbyte;
}

// Read 2 ASCII Hex CRC characters and return as byte
int checkcrc()
{
 	unsigned char crcbyte,strg[3];
	
 	strg[0] = getchar();  if(intelhexecho) putchar(strg[0]); 
	strg[1] = getchar();  if(intelhexecho) putchar(strg[1]);
	strg[2] = 0;
	crcbyte = hextobyte(strg);
 	return crcbyte;
}

//  Read for ASCII hex characters and convert to int address
unsigned int getinteladdress()
{
	unsigned char strg[3],addressbyte;
	unsigned int address;
	
	addressbyte = gethexbyte();
	address = gethexbyte();
	address += addressbyte*256;
	return address;
}

// Convert Intel Hex ASCII string to binary
// :10 0100 00 0C948F0D0C941D0E0C941D0E0C941D0E 38
// :1001000000C941D0E0C941D0E0C944E590C941D0E38
//  This routine decodes a line of Intel Hex - support
//  for extended address and end of file records is included
int DecodeIntelHex()
{
 	 unsigned char strg[8];
 	 unsigned char addressbyte,inbyte;
	 int i;
	 
	 // Reset CRC
	 crc = 0;
	 endofintelhex = NO;
	 
	 // Convert count of bytes on line
	 bytesonline = gethexbyte();
	 
	 // Convert address to binary - ignore filler byte
	 // Typical Intel Hex consists of 4 ASCII characters
	 inteladdress = getinteladdress();
	 
	 // Read record type (2 characters)
	 recordtype = gethexbyte();
	 
	 // Check the record type and process accordingly
	 // If record type is extended address (bits 31-16) convert to binary
	 if(recordtype == 4)  extendedaddress = getinteladdress();
	 
	 // If end of file record reset extended address to zero
	 else if(recordtype == 1)  {
	 	extendedaddress = 0;
	 	endofintelhex = YES;
//	 	puts("End of Hex record found\n\r");
	 	}
	 
	 // else must be standard data record
	 else {
	 	for(i=0; i<bytesonline; i++) intelbinary[i] = gethexbyte();
	 	}
		 
	// Check CRC - todo
	if(checkcrc() == crc) return 1;
	else return -1;
}

// Load Intel Hex file into SDRAM
void ProgramSDRAM()
{
     int i;
	 
	 for(i=0; i<bytesonline; i++)  WriteBlackfinMemory(inteladdress+i,intelbinary[i]);
}

//  Read the status of Flash chip 1 or 2
unsigned char FlashStatus(int FlashChip)
{
	unsigned char flashstatus;
	
	FlashChipSelect(FlashChip,LOW);
	
	//  Flash Status
	SPI_MasterTransmit(0x57);
	flashstatus = SPI_MasterReceive();
	
	FlashChipSelect(FlashChip,HIGH);
	
	return flashstatus;
}

//  Transfer contents of a Flash page to buffer 1 or 2
void FlashPageToBuffer(int FlashChip,int page, int bufferno)
{
	int i;
	
	FlashChipSelect(FlashChip,LOW);
	
	if(bufferno==1) SPI_MasterTransmit(0x53);
	else SPI_MasterTransmit(0x55);

	// Convert address to 24 bit address
	ConvertFlashAddress(page*264);
	for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);		
	FlashChipSelect(FlashChip,HIGH);	

	//  Wait until erase finishes
	while(!(FlashStatus(1)&0x80))  ;
}

// Erase a Page and wait until done
void FlashPageErase(int FlashChip,int page)
{
	int i;
	
	FlashChipSelect(FlashChip,LOW);
	
	//  Erase a Flash page
	SPI_MasterTransmit(0x81);
	 
	// Convert address to 24 bit address
	ConvertFlashAddress(page*264);
	for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);	
	
	FlashChipSelect(FlashChip,HIGH);
	
	//  Wait until erase finishes
	while(!(FlashStatus(1)&0x80))  ;
	
}

//  Transfer contents of buffer 1 or 2 to Flash Page
//  Wait until done
void FlashPageWrite(int FlashChip,int page,int bufferno)
{
	int i;
	
	FlashChipSelect(FlashChip,LOW);
	
	//  Erase a Flash page
	if(bufferno==1)  SPI_MasterTransmit(0x88);
	else SPI_MasterTransmit(0x88);
	 
	// Convert address to 24 bit address
	ConvertFlashAddress(page*264);
	for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);	
	
	FlashChipSelect(FlashChip,HIGH);
	
	//  Wait until erase finishes
	while(!(FlashStatus(1)&0x80))  ;
}

//  Write bytes to Flash buffer
//    Note this routine does not check for wrap around - calling routine responsibility
//    This routine loads 16 bytes at a time
void FlashBytesToBuffer(int bufferno)
{
     int i;

     // Flash CS low	 
	 FlashChipSelect(1,LOW);
	 
	 //  Buffer write
	 if(bufferno==1) SPI_MasterTransmit(0x84);
	 else SPI_MasterTransmit(0x87);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 for(i=0; i<bytesonline; i++)  SPI_MasterTransmit(intelbinary[i]);
	 
	 // Flash CS high
	 FlashChipSelect(1,HIGH);
}

// Write a single byte to a Flash Page - compound operation
void WriteFlashByte(int FlashChip,unsigned int address,unsigned char inbyte)
{
	int page;
	
	page = address/264;
	
	// Load Flash Page to buffer 1
	FlashPageToBuffer(FlashChip,page,1);
	
	// Erase page
	FlashPageErase(FlashChip,page);
	
	// Write byte to buffer 1
	inteladdress = address%264;
	intelbinary[0] = inbyte;
	bytesonline = 1;
	FlashBytesToBuffer(1);
	
	// Program Flash Page
	FlashPageWrite(FlashChip,page,1);
}

// Write Flash by passing data through buffers
void ProgramFlashPage(int FlashChip,unsigned int address,int length,int bufferno)
{
	 int i;

     // Flash CS low	 
	 FlashChipSelect(1,LOW);
	 
	 //  Buffer write then page program
	 if(bufferno==1) SPI_MasterTransmit(0x82);
	 else SPI_MasterTransmit(0x85);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 for(i=0; i<bytesonline; i++)  SPI_MasterTransmit(intelbinary[i]);
	 
	 // Flash CS high
	 FlashChipSelect(1,HIGH);
}

//  Flash initialization and control paramters
int currentflashbyteoffset;  // Current byte in flash RAM buffer
int currentflashpage;   // Current flash page being loaded
int currentflashchip;   // Current flashchip being accessed - 1 or 2
char flashbufferloaded;  // Status of flash buffer load - YES/NO
char flashpageprogrammed; // Status of flash page programming - YES/NO
char flashbufferempty;   // True if data loaded into flash RAM buffer but not programmed

//  Reset Flash Variables for next Intel Hex download
void   ResetFlashVars()
{
	currentflashbyteoffset = 0;  // Current byte in flash RAM buffer
	currentflashpage = 0;   // Current flash page being loaded
	currentflashchip = 1;   // Current flashchip being accessed - 1 or 2
	flashbufferloaded = NO;  // Status of flash buffer load - YES/NO
	flashpageprogrammed = NO; // Status of flash page programming - YES/NO
	flashbufferempty = YES;
	extendedaddress = 0;
	endofintelhex = YES;
}

//  Flash programming routine 
//     
//  When loading Intel Hex via the serial line this routine loads each line into the
//  Flash's RAM buffer so main memory is not used. As soon as sufficient data to fill 
//  one page is received the page is written.  The page is also written if the next
//  line contains data for a new page (or flash chip).
//
//  The Flash device contains two SRAM buffers of 264 or 528 bytes and
//  the following flash pages:
//      AT45DB041 - 2048 pages -  540672 bytes (264 byte)
//      AT45DB081 - 4096 pages - 1081344 bytes (264 byte)
//      AT45DB161 - 8192 pages - 2162688 bytes (528 byte)
//  Each Flash page takes 8 msec to erase and 12 msec to program or
//  20 msec to erase/program.  Each page is a certain length - the programming
//  process must wait for each page to be programmed. Note if Intel Hex is loaded
//  using fast serial then XON/XOFF flow control must be enabled in the serial
//  driver or an overrun condition will occur.
void ProgramFlash()
{
	unsigned char outbyte;
	int i,chipselect,bytesinpage,fpagesize,fbytesize,loopcnt=0,addressinpage;
	unsigned int address,bytesleftinpage;
	long superaddress,endaddress,storageendaddress;

	// Program flash page
	if(endofintelhex==YES) {
		if(flashpageprogrammed==NO) {
	 		FlashPageWrite(currentflashchip,currentflashpage,1);
	 		Delay(10000);
	 		flashpageprogrammed = YES;
	 		puts("Last page programmed\n\r"); 
			}
		ResetFlashVars();
		}
	// Load data and program flash page if RAM buffer full
	else {
		//  Convert Intel Address (with extended address) to RAM buffer address
		superaddress = extendedaddress*65536 + inteladdress;
		endaddress = superaddress + bytesonline - 1;
	
	puts("\n\r");
	puts("superaddress="); printlonghex(superaddress); puts("\n\r");
	puts("endaddress="); printlonghex(endaddress); puts("\n\r");
	
		// Calculate which Flash Chip to program 
		//   Take care of traverses from one Flash chip to the next
		if(superaddress<f1bytesize) {
			currentflashchip = 1;
			fpagesize = f1pagesize;
			fbytesize = f1bytesize;
			if(endaddress > (f1bytesize+f2bytesize))  endaddress = f1bytesize+f2bytesize;
			}
		// Start programming second Flash chip
		else if((superaddress>f1bytesize)&&(superaddress<(f1bytesize+f2bytesize))) {
			currentflashchip = 2;
			fpagesize = f2pagesize;
			fbytesize = f2bytesize;
			superaddress -= f1bytesize;
			endaddress -= f1bytesize;
			if(endaddress > (f1bytesize+f2bytesize)) endaddress = f2bytesize;
			}
		else currentflashchip = 3;  // Start range outside of Flash Chips

		if(currentflashchip<3) {
			currentflashpage = superaddress/fpagesize;
			addressinpage = superaddress%fpagesize;
			
	puts("addressinpage="); printint(addressinpage,5,0); puts("\n\r");

			if(bytesonline<(fpagesize-addressinpage)) bytesinpage = bytesonline;
			else bytesinpage = fpagesize-addressinpage;
			
	puts("currenflashpage="); printint(currentflashpage,5,0); puts("\n\r");
	puts("bytesinpage 2="); printint(bytesinpage,5,0); puts("\n\r");
	puts("bytesonline="); printint(bytesonline,5,0); puts("\n\r");
	
//			bytesonline -= bytesinpage;
			
	puts("bytesonline 2="); printint(bytesonline,5,0); puts("\n\r");
		
			loopcnt = 0;
			bytesleftinpage = 0;
	    	do  {

	    		// Load flash page if not loaded and erase page
	    		if(flashbufferloaded==NO) {
	    			FlashPageToBuffer(currentflashchip,currentflashpage,1);
	    			FlashPageErase(currentflashchip,currentflashpage);
	    			Delay(10000);  // delay 10msec
	    			flashbufferloaded = YES;
	    			flashpageprogrammed = NO;
	    			}

     			// Flash CS low	 
	 			FlashChipSelect(currentflashchip,LOW);
	 
	 			//  Put data in Flash RAM Buffer 1
	 			SPI_MasterTransmit(0x84);
	 
	 			// Convert address to 24 bit address
	 			ConvertFlashAddress(addressinpage);
	 		
//	 			puts("\n\r");
//	 			puts("Flash byte address=");
//	 			bytetohex(FlashAddress[0]); puts(hexbyte);
//	 			bytetohex(FlashAddress[1]); puts(hexbyte);
//	 			bytetohex(FlashAddress[2]); puts(hexbyte);
//	 			puts("\n\r");
//	 			puts("Current flash page="); printint(currentflashpage,5,0); puts("\n\r");
//				puts("bytesinpage="); printint(bytesinpage,5,0); puts("\n\r");
//				puts("bytesonline="); printint(bytesonline,5,0); puts("\n\r");
	 		
	 			for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 			for(i=0; i<bytesinpage; i++)  SPI_MasterTransmit(intelbinary[i+bytesleftinpage]);
	 
	 			// Deselect flash chip
	 			FlashChipSelect(currentflashchip,HIGH);
	 		
	 			//  Write Buffer to Flash page if bytesonline exceeded page boundary
	 			if((addressinpage+bytesonline)>fpagesize)  {
	 				FlashPageWrite(currentflashchip,currentflashpage,1);
	 				Delay(10000);
	 				flashpageprogrammed = YES;
	 				flashbufferloaded = NO;
	 				currentflashpage++;
	 				addressinpage = 0;
	 				}
	 				
	 			//  Calculate next index parameters
	 			superaddress += bytesinpage;
				puts("superaddress="); printlonghex(superaddress); puts("\n\r");
			
	 			//  Check for Chip traverse - reset address if going to next chip
	 			//   TODO - doesn't handle traverse to next chip
				
	 			bytesleftinpage = bytesinpage;
				bytesinpage = bytesonline - bytesinpage;
				bytesonline = 0;		
	 		
	        	} while((superaddress<=endaddress)&&(loopcnt++<40));  // loopcnt is escape
			}
		}	
}

void DumpDecodedIntelHex()
{
	int i;
	
	puts("Dump IntelHex ");
	printint(inteladdress,4,' ');
	puts(" len ");
	printint(bytesonline,3,' ');
	putchar(' ');
	for(i=0; i<bytesonline; i++)  {
		bytetohex(intelbinary[i]);
		puts(hexbyte);
		}
	puts("\n\r");

}


void Init_Flash()
{
	currentflashbyteoffset = 0;
	currentflashpage = 0;
	currentflashchip = 1;
	flashbufferloaded = NO;
	flashpageprogrammed = NO;
	
}

//  Fill Flash Memory with a byte value or from SDRAM
//   If length is longer than available storage program upto available storage
//   Other routine checks Flash configuration
void FillFlashMemory(unsigned int address,long length,unsigned char databyte,unsigned char fillmode)
{
	unsigned char outbyte;
	int i,chipselect,bytesinpage,fpagesize,fbytesize,loopcnt,addressinpage;
	long superaddress,endaddress,storageendaddress;
	
	superaddress = extendedaddress*65536 + address;
	endaddress = superaddress + length - 1;
	
//	puts("\n\r");
//	puts("superaddress="); printlonghex(superaddress); puts("\n\r");
//	puts("endaddress="); printlonghex(endaddress); puts("\n\r");
	
	// Calculate which Flash Chip to program 
	//   Take care of traverses from one Flash chip to the next
	if(superaddress<f1bytesize) {
		chipselect = 1;
		fpagesize = f1pagesize;
		fbytesize = f1bytesize;
		if(endaddress > (f1bytesize+f2bytesize))  endaddress = f1bytesize+f2bytesize;
		}
	// Start programming second Flash chip
	else if((superaddress>f1bytesize)&&(superaddress<(f1bytesize+f2bytesize))) {
		chipselect = 2;
		fpagesize = f2pagesize;
		fbytesize = f2bytesize;
		superaddress -= f1bytesize;
		endaddress -= f1bytesize;
		if(endaddress > (f1bytesize+f2bytesize)) endaddress = f2bytesize;
		}
	else chipselect = 3;  // Start range outside of Flash Chips

	if(chipselect<3) {
		currentflashpage = superaddress/fpagesize;
		addressinpage = superaddress%fpagesize;
//	puts("addressinpage="); printint(addressinpage,5,0); puts("\n\r");
		if((addressinpage==0)&&(length>=fpagesize))  bytesinpage = fpagesize;
		else if(length<(fpagesize-addressinpage)) bytesinpage = length;
		else bytesinpage = fpagesize-addressinpage;
//	puts("currenflashpage="); printint(currentflashpage,5,0); puts("\n\r");
//	puts("bytesinpage 2="); printint(bytesinpage,5,0); puts("\n\r");
//	puts("length="); printlonghex(length); puts("\n\r");
		length -= bytesinpage;
//	puts("length 2="); printlonghex(length); puts("\n\r");
		
		loopcnt = 0;
	    do  {

	    	if(bytesinpage<fpagesize) FlashPageToBuffer(chipselect,currentflashpage,1);
	    	FlashPageErase(chipselect,currentflashpage);
	    	Delay(10000);
	    	
	    	// Fill buffer with byte
     		// Flash CS low	 
	 		FlashChipSelect(chipselect,LOW);
	 
	 		//  Buffer 1 write
	 		SPI_MasterTransmit(0x84);
	 
	 		// Convert address to 24 bit address
	 		//  TODO - convert this to superaddress 
	 		extendedaddress = superaddress/65536;
	 		address = superaddress - extendedaddress*65536;
	 		ConvertFlashAddress(address);
	 		
	 		for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 		for(i=0; i<bytesinpage; i++)  {
	 			if(fillmode==FROMSDRAM)  SPI_MasterTransmit(*(pSDRAMBot+fillindex++));
	 			else SPI_MasterTransmit(databyte);
	 			}
	 
	 		// Flash CS high
	 		FlashChipSelect(chipselect,HIGH);
	 		
	 		//  Write Buffer to Flash page
	 		FlashPageWrite(chipselect,currentflashpage,1);
	 		Delay(10000);
	 		
	 		//  Calculate next index parameters
	 		superaddress += bytesinpage;
//			puts("superaddress="); printlonghex(superaddress); puts("\n\r");
			
	 		//  Check for Chip traverse - reset address if going to next chip
	 		if((chipselect==1)&&(superaddress>=f1bytesize)) superaddress = 0;
			currentflashpage = superaddress/fpagesize;
			addressinpage = superaddress%fpagesize;
			if((addressinpage==0) && (length>=fpagesize))  bytesinpage = fpagesize;
			else if(length<(fpagesize-addressinpage)) bytesinpage = length;
			else bytesinpage = fpagesize-addressinpage;
			length -= bytesinpage;	 		
	 		
	        } while(superaddress<=endaddress);
		    
		}
}

//  Called by ':' command from serial port 
//
//  Programming data fed line by line.  For SDRAM program each line
//  as it is received - for Flash accumulate enough data to fill 
//  a buffer then program buffer.  Note XON/XOFF flow control should 
//  be enabled when programming flash due to the 20msec burn time
//  per 264 byte buffer.
void ProgramMemory()
{
	if(endofintelhex==YES)  puts("\n\r");
	if(intelhexecho==YES)  putchar(':');
	endofintelhex = NO;
 	DecodeIntelHex();
	switch (programmode) {
		case LBlackfinSDRAM:
		    ProgramSDRAM();
//		    DumpDecodedIntelHex();
		    // Debug output - one . for every line
//		    putchar('.');
			break;
		case LFLASH:
		    ProgramFlash();
			break;
		} 
}

// Read an integer from serial and convert to word
int getint()
{
	char instrg[10],inchar;
	int i;
	
	for(i=0; i<10; i++)  {
		inchar = getchar();
		if((inchar==' ')||(inchar==CR)||(inchar==',')||(inchar==LF)) {
			instrg[i] = 0x00;
			break;
			}
		instrg[i] = inchar;
		putchar(inchar);
		}
	putchar(inchar);
	return atoi(instrg);
}

//  Read ASCII hex value and convert to byte
unsigned char gethextobyte()
{
	unsigned char hexbyte;
	unsigned char instrg[10],inchar;
	int i;
	
	for(i=0; i<5; i++)  {
		inchar = getchar();
		if((inchar == CR)||(inchar==LF)) {
			instrg[i] = 0x00;
			break;
			}
		instrg[i] = inchar;
		putchar(inchar);
		}
	puts("\n\r");
	for(i=0; i<2; i++) instrg[i] = instrg[i+2];
	instrg[2] = 0;
	return hextobyte(instrg);
}

// Fill SDRAM or SRAM with a byte value - enter base address and length
void FillSDRAMMemory(unsigned int address,long length,unsigned char databyte)
{
	int i;
	
	i = 0;
	while(length-->=0) *(pSDRAMBot+address+extendedaddress*65536+i++) = databyte;
}


//  Fill Blackfin SDRAM with a byte from start address of length
//
//  TODO - fill Flash
void FillMemoryCommand()
{
	unsigned char databyte,inchar;
	unsigned int address;
	unsigned long longaddress,length;
	
	// Fill Flash or SDRAM?
	inchar = getchar();
	putchar(inchar);
	puts("\n\rFill Memory: enter address, length, byte (hex): "); 
	// Read address and convert to int address and extended address
	longaddress = getlonghex();
	extendedaddress = longaddress/65536;
	address = longaddress - extendedaddress*65536;
	length = getlonghex();
	databyte = getlonghex();
	if(inchar=='s'||inchar=='S') FillSDRAMMemory(address,length,databyte);
	else if(inchar=='f'||inchar=='F') FillFlashMemory(address,length,databyte,FROMCONST);
	puts("\n\r");
}


void LoadMemory()
{
    unsigned char inchar;
	
	inchar = getchar();
	putchar(inchar);
	switch (inchar) {
		case 'f': case 'F':
//			putchar(inchar);
		    puts("\n\rSet Load Flash as default\n\r");
			programmode = LFLASH;
		    break;
		case 's': case 'S':
//			putchar(inchar);
		    puts("\n\rSet Load SDRAM as default\n\r");
			programmode = LBlackfinSDRAM;
		    break;
		default:
		   break;
	    }
}

//  Main memory to memory move command
//
//    Moves to/from Flash require special procedures
void MoveMemory(char source, char destination,long saddress,long length,long daddress)
{
	int i;
	
	if(source=='S') source = 's';
	if(destination=='F') destination = 'f';
	
	//  SDRAM to SDRAM - direct move
	if(source=='s'&&destination=='s') {
		puts("Transfer from SDRAM to SDRAM\n\r");
		for(i=0; i<length; i++)  *(pSDRAMBot+i+daddress) = *(pSDRAMBot+i+saddress);
		}
	//  SDRAM to Flash - Flash writes must be handled in blocks
	else if(source=='s'&&destination=='f') {
		puts("Transfer from SDRAM to Flash\n\r");
		fillindex = saddress;
		FillFlashMemory(daddress,length,0x00,FROMSDRAM);
		}
	//  Flash to SDRAM - TODO add Flash chip traverse
	else if(source=='f'&&destination=='s') {
		FlashChipSelect(1,LOW);
		SPI_StartReadFlashPage(saddress);  // continuous read command 0x68
		while(length--) *(pSDRAMBot+daddress++) = SPI_MasterReceive();
		puts("\n\rlast address=");	printlonghex(pSDRAMBot+daddress-1);
		puts("\n\r");
		FlashChipSelect(1,HIGH);
		}
	//  Flash to Flash - Flash writes must be handled in blocks
	else if(source=='f'&&destination=='f') {
		puts("Function not yet implemented\n\r");
		}
}

//  Memory to memory move command parser
void MoveMemoryCommand()
{
	char source,destination;
	long sourceaddress,destinationaddress,length;
	
	puts("\n\rEnter source and destination memory [S|F-S|F]: ");
	source = getchar();
	putchar(source);
	destination = getchar();
	putchar(destination);
	puts("\n\rEnter source address, length and destination address: ");
	sourceaddress = getlonghex();
	length = getlonghex();
	destinationaddress = getlonghex();
	puts("\n\r");
	MoveMemory(source,destination,sourceaddress,length,destinationaddress);	
}

// Output program version number
void ProgramVersion()
{
      char majorversion[3],minorversion[4];
	  
	  puts("\n\rP1 Pirana Monitor Version ");
	  printint(MAJORVER,1,0); putchar('.');
	  printint(MINORVER,2,0);
	  puts("\n\r");
}


//  Using 118.8MHz SCLK counter clocks 119 counts per microsecond
//  Maximum count period is 32 seconds
void CountPeriodStart(void)
{
	unsigned long countperiod;
	
	countperiod = (long) 32000000;
	countperiod *= 131;
	//  Use timer 2 as count up timer
	*pTIMER2_CONFIG	= 0x0049;
	*pTIMER2_PERIOD	= countperiod;
	*pTIMER2_WIDTH = 0x00000006;
	*pTIMER_ENABLE |= 0x0004;
	// Timer stops once period count is reached
	*pTIMER_DISABLE |= 0x0004;
}

//  Return count period in microseconds and reset counter
long CountPeriod()
{
	long countperiod;
	
	countperiod = *pTIMER2_COUNTER/131;
	*pTIMER_DISABLE &= 0x000B;	
	*pTIMER_ENABLE &= 0x000B;
	*pTIMER_STATUS |= 0x4000;
	return countperiod;
}

//  Return count period in microseconds and reset counter
long CountUPeriod()
{
	long countperiod;
	
	countperiod = *pTIMER2_COUNTER;
	*pTIMER_DISABLE &= 0x000B;	
	*pTIMER_ENABLE &= 0x000B;
	*pTIMER_STATUS |= 0x4000;
	return countperiod;
}


//  Exercise most of the Blackfin functions
void TestBlackfin(void)
{
	unsigned int temp,temp2;
	int failcnt;
	long i,badmatchcnt,bankcnt,bankbadcnt;
	
	puts("\n\rTest P1 Blackfin\n\r");
	puts(" Test RAM");
		puts(" - Passed\n\r");
		
	//  Test SDRAM by writting 0x55 and 0xAA to all locations
	//     Non-destructive test
	puts(" Test SDRAM");
	badmatchcnt = 0;
	bankcnt = 0;
	// Byte access
	puts("\n\r  Byte Access test");
	while(bankcnt<512) {
		bankbadcnt = 0;
	    for(i=bankcnt*65536; i<bankcnt*65536+65536; i++) {
		    temp = *(pSDRAMBot+i);
		    *(pSDRAMBot+i) = 0x55;
		    if(*(pSDRAMBot+i)!=0x55) bankbadcnt++;
		    *(pSDRAMBot+i) = 0xAA;
		    if(*(pSDRAMBot+i)!=0xAA) bankbadcnt++;
		    *(pSDRAMBot+i) = temp;
		    }
		bankcnt++;
		badmatchcnt += bankbadcnt;		
	    if(bankbadcnt) {
	    	puts("\n\r  Bank ");  printint(bankcnt,4,0); putchar(' ');
			puts(" failed with ");
			printint(bankbadcnt,5,0);
			puts(" mismatches");
			}
		bankbadcnt = 0;
		}
	//  Output results
	if(badmatchcnt) {
		puts("\n\r  SDRAM failed with ");
		printint(badmatchcnt,6,0);
		puts(" mismatches\n\r");
		}
	else puts(" - Passed\n\r");
	// Word access
	puts("  Word Access test");
	badmatchcnt = 0;
	bankcnt = 0;
	while(bankcnt<256) {
		failcnt = 0;
		bankbadcnt = 0;
	    for(i=bankcnt*65536; i<bankcnt*65536+65536; i++) {
		    temp = *(pSDRAMWordBot+i);
		    *(pSDRAMWordBot+i) = 0x5555;
		    temp2 = *(pSDRAMWordBot+i);
		    if(temp2 != 0x5555) {
		    	bankbadcnt++;
		    	failcnt++;
		    	if((failcnt<1100)&&(bankcnt==255)) {
		    		puts("\n\r  Failed 5 at ");
		    		printint(i%65536,5,0);
		    		puts(" wrote 0x5555 - read ");
		    		*(pSDRAMWordBot+i) = 0x5555;
		    		temp2 = *(pSDRAMWordBot+i);
		    		addresstohex(temp2);
		    		puts(hexaddress);
		    		}
		    	}
		    *(pSDRAMWordBot+i) = 0xAAAA;
		    if(*(pSDRAMWordBot+i)!=0xAAAA) {
		    	bankbadcnt++;
		    	failcnt++;
		    	if((failcnt<1100)&&(bankcnt==255)) {
		    		puts("\n\r  Failed A at ");
		    		printint(i%65536,5,0);
		    		}
		    	}
		    *(pSDRAMWordBot+i) = temp;
		    }
		bankcnt++;
		badmatchcnt += bankbadcnt;		
	    if(bankbadcnt) {
	    	puts("\n\r  Bank ");  printint(bankcnt,4,0); putchar(' ');
			puts(" failed with ");
			printint(bankbadcnt,5,0);
			puts(" mismatches");
			}
		bankbadcnt = 0;
		}
	//  Output results
	if(badmatchcnt) {
		puts("\n\r  SDRAM failed with ");
		printint(badmatchcnt,5,0);
		puts(" mismatches\n\r");
		}
	else puts(" - Passed\n\r");

	
	//  Test Flash 
	puts(" Test Flash");
		puts(" - Passed\n\r");
	puts(" Test UART");
		puts(" - Passed\n\r");
	puts(" Test SPORTs");
		puts(" - Passed\n\r");
	puts(" Test SPI");
		puts(" - Passed\n\r");
	puts(" Test CM64");
		puts(" - Passed\n\r");
}

void BootApplication()
{
	unsigned long bootaddress;
	
	puts("\n\rAddress of Application to Boot [32bit hex]: ");
	bootaddress = getlonghex();
	
	//  Pass to .ldr loader program
	//    The boot image must conform to the Analog Device Blackfin .ldr format
	//    This process may overwrite the boot monitor
	//    Registers are loaded onto the stack - if the new application executes 
	//    an exit or return it should come back to the BootMonitor but this 
	//    is not guaranteed.
	LoadElfImage(bootaddress);
	
}

void ExtendedMessage()
{
	puts("\n\rP1 Pirana Extended Commands\n\r\n\r"); 
	puts("   i  - 2000 init timing\n\r");
	puts("   g  - Demo of I2C init\n\r");
	puts("   0  - Flash Status\n\r");
	puts("   1  - Write byte to CM64\n\r");
	puts("   2  - Init CM64\n\r");
	puts("   3  - Halt video DMA/PPI operation\n\r");
	puts("   4  - YUV to RGB timing test\n\r");
	puts("   5  - Read SAA7113 ID continuously\n\r");
	puts("   6  - Read SAA7113 status continuously\n\r");
	puts("   7  - Reset SAA7113 continuously\n\r");
	puts("   8  - SPI transmit test\n\r");
	puts("   9  - SAA7113 Status\n\r");
	puts("   H  - Help Menu\n\r");
}


// Output default P1 Blackfin Sign-on message
void SignOnMessage()
{
	puts("\n\rP1 Pirana Blackfin Monitor v"); 
		printf(MAJORVER,2,0); putchar('.'); printint(MINORVER,2,0); 
		puts("\n\r");
	puts("Copyright 2004-2005, SOC Machines, Inc.\n\r\n\r");
	puts("   D[S|F]  - Display [S]DRAM or [F]lash memory\n\r");
	puts("   L[S|F]  - Set default memory type for Intel Hex Load to [S]DRAM or [F]lash\n\r");
	puts("   F[S|F]  - Fill [S]DRAM or [F]lash memory [address,length,byte]\n\r");
	puts("   M[S/F|S/F]  - Move [S]DRAM/[F]lash memory to [S]DRAM/[F]lash\n\r");
	puts("                 [source address,length,destination address]\n\r");
	puts("   K   - Compare memory (range)\n\r");
	puts("   T   - Test Blackfin\n\r");
	puts("   B   - Boot Application (address,.ldr format)\n\r");
	puts("   P   - Display/Set Preferences\n\r");
	puts("   C   - Display/Set Blackfin Clock\n\r");
	puts("   X   - Display/Set new Internet Settings\n\r");
	puts("   V   - Version\n\r");
	puts("   S   - P1 Specifications\n\r");
	puts("   E   - Extended commands\n\r");
	puts("   H   - Help Menu\n\r");
}

//  Convert 4 byte binary IP value to ASCII format
void DecodeIP()
{
	int i;
	
	for(i=0; i<3; i++)  {
		printint(ip[i],3,0);
		putchar('.');
		}
	printint(ip[3],3,0);
}

//  Convert IP ASCII string to 4 byte binary value
void EncodeIP(char *instrg)
{
	char strg[4];
	int i,ptr,cnt;
	
	cnt = 0;
	ptr = 0;

	while(cnt<3) {
		i = 0;
	    while((strg[i]=instrg[ptr])!='.')  {
		    i++;
		    ptr++;
	    	}
	    strg[i] = 0;
	    i++;
	    ptr++;
	    ip[cnt] = atoi(strg);
	    cnt++;
		}
	i = 0;
	while((strg[i]=instrg[ptr])!=0)  {
		i++;
		ptr++;
		}
	strg[i] = 0;
	ip[3] = atoi(strg);	
}

//  Convert Ethernet MAC ASCII address to binary string
void EncodeMAC(char *instrg)
{
	unsigned char strg[3];
	int i,ptr;
	
	ptr = 0;
	strg[2] = 0;
	for(i=0; i<12; i+=2) {
		strg[0] = instrg[i];
		strg[1] = instrg[i+1];
		ethernetmac[ptr++] = hextobyte(strg);
		}
}

//  Convert Ethernet MAC address binary string the ASCII string
void DecodeMAC()
{
	int i;
	
	for(i=0; i<6; i++)  {
		bytetohex(ethernetmac[i]);
		puts(hexbyte);
		}
}

//  Check for Ethernet settings stored in AVR else use
//  startup defaults
void Init_Internet()
{
	int i;
	
	EncodeMAC(ETHERNETMACID); 
	EncodeIP(IPADDRESS);  for(i=0; i<4; i++)  ipaddress[i] = ip[i];
	EncodeIP(IPMASK);     for(i=0; i<4; i++)  ipmask[i] = ip[i];
	EncodeIP(IPGATEWAY);  for(i=0; i<4; i++)  ipgateway[i] = ip[i];
}

char *getstrg()
{
	char inchar;
	static char *instrg = "                                        ";
	int i;
	
	i = 0;
	inchar = ' ';
	instrg[0] = 0;
	while((inchar!=CR)&&(inchar!=LF)) {
		inchar = getchar();
		putchar(inchar);
		if((inchar!=CR)&&(inchar!=LF))  instrg[i++] = inchar;
		}
	instrg[i] = 0;
	return instrg;
}

void InternetSettings()
{
	int i;
	
	Init_Internet();
	puts("\n\rInternet Settings\n\r");
	puts("  Ethernet MAC: "); DecodeMAC(); puts("\n\r");
	puts("  IP Address:   "); 
		for(i=0; i<4; i++)  ip[i] = ipaddress[i];
		DecodeIP(); puts("\n\r");
	puts("  IP Mask:      "); 
	    for(i=0; i<4; i++)  ip[i] = ipmask[i];
	    DecodeIP(); puts("\n\r");
	puts("  IP Gateway:   "); 
	    for(i=0; i<4; i++)  ip[i] = ipgateway[i];
	    DecodeIP(); puts("\n\r");
	puts("  User ID:      "); puts(USERID); puts("\n\r");
	puts("  Password:     "); puts(PASSWORD); puts("\n\r\n\r");
}

void GetInternetSettings()
{
	char *instrg;
	int i;
	
	//  Display/change Ethernet MAC Address
	puts("\n\rEthernet MAC ("); DecodeMAC(); puts("): ");
	instrg = getstrg();
	puts("\n\r");	
	if(instrg[0])  EncodeMAC(instrg); 
		
	//  Display/change IP address
	puts("IP address ("); 
	for(i=0; i<4; i++)  ip[i] = ipaddress[i];
	DecodeIP(); puts("): ");
	instrg = getstrg();
	puts("\n\r");
	if(instrg[0])  {
		EncodeIP(instrg);  
		for(i=0; i<4; i++)  ipaddress[i] = ip[i];
		}
	
	//  Display/change IP mask
	puts("Mask ("); 
	for(i=0; i<4; i++)  ip[i] = ipmask[i];
	DecodeIP(); puts("): ");
	instrg = getstrg();
	puts("\n\r");	
	if(instrg[0])  {
		EncodeIP(instrg);     
		for(i=0; i<4; i++)  ipmask[i] = ip[i];
		}
	
	//  Display/change IP gateway
	puts("Gateway ("); 
	for(i=0; i<4; i++)  ip[i] = ipgateway[i];
	DecodeIP(); puts("): ");
	instrg = getstrg();
	puts("\n\r");	
	if(instrg[0])  {
		EncodeIP(instrg);  
		for(i=0; i<4; i++)  ipgateway[i] = ip[i];
		}
}

void CheckFlash()
{
	unsigned char inbyte;
	
	inbyte = FlashStatus(1);
	if((inbyte&FLASHIDMASK)==AT45DB041ID) {
		f1pagesize = 264;
		f1bytesize = 540672;
		}
	else if((inbyte&FLASHIDMASK)==AT45DB081ID) {
		f1pagesize = 264;
		f1bytesize = 540672*2;
		}	
	else if((inbyte&FLASHIDMASK)==AT45DB161ID) {
		f1pagesize = 528;
		f1bytesize = 540672*4;
		}
	else {
		f1pagesize = 0;
		f1bytesize = 0;
		}
	inbyte = FlashStatus(2);
	if((inbyte&FLASHIDMASK)==AT45DB041ID) {
		f2pagesize = 264;
		f2bytesize = 540672;
		}
	else if((inbyte&FLASHIDMASK)==AT45DB081ID) {
		f2pagesize = 264;
		f2bytesize = 540672*2;
		}	
	else if((inbyte&FLASHIDMASK)==AT45DB161ID) {
		f2pagesize = 528;
		f2bytesize = 540672*4;
		}
	else {
		f2pagesize = 0;
		f2bytesize = 0;
		}		
}

//  Compare Memory location range
void CompareMemory()
{
	long sourceaddress,destinationaddress,length,i,mismatchcount;
	
	mismatchcount = 0;
	puts("\n\rEnter source address, length and destination address: ");
	sourceaddress = getlonghex();
	length = getlonghex();
	destinationaddress = getlonghex();
	puts("\n\r");
	for(i=0; i<length; i++) {
		if(*(pSDRAMBot+i+sourceaddress) != *(pSDRAMBot+i+destinationaddress))  {
			puts("Mismatch at address: ");
			PrintHexWord(i+sourceaddress);
			puts("\n\r");
			mismatchcount++;
			}
		}
	if(mismatchcount) {
		printint(mismatchcount,6,' '); 
		puts(" mismatches");
		puts("\n\r");
		}
	else puts("No mismatches\n\r");	
}

//  Display/set user preferences
//  For the moment do an SDRAM memory compare
void Preferences()
{
	char inchar;
	long sourceaddress,destinationaddress,length,i,mismatchcount;

	puts("\n\rPreference Commands\n\r\n\r"); 
	puts("   I[S|L]  - Intel Hex [S]hort/[L]long format\n\r");
	puts("   H  - Help Menu\n\rp-");
	
	inchar = getchar();
	putchar(inchar);
	switch (inchar) {
		case 'i': case 'I':
			inchar = getchar();
			putchar(inchar);
			if((inchar=='s')||(inchar=='S')) {
				intelhexprintmode = SHORTMODE;
				puts("  Intel Hex short mode\n\r");
				}
			else if((inchar=='l')||(inchar=='L')) {
				intelhexprintmode = LONGMODE;
				puts("  Intel Hex long mode\n\r");
				}
			break;
		case 'h': case 'H':
			SignOnMessage(); 
			putchar('>');
			break;
		}	
}

// Output characteristics of this P1
void Specifications()
{	
	puts("\n\rP1 Specifications:\n\r");
	puts("  DSP Processor:  BF532 Blackfin, 394MHz, ");
	puts("Rev 0."); printint((*pDSPID&0x0000000F),2,0); puts("\n\r");
	puts("  SDRAM Memory:   16Mx16 130MHz\n\r");
	puts("  Flash:          ");
	if(f1pagesize>0&&f2pagesize>0) puts("2 Flash chips, ");
	else puts("1 Flash chip ");
	if(f1bytesize==540672) puts("512Kx8 ");
	else if(f1bytesize==540672*2) puts("1Mx8 ");
	else if(f1bytesize==540672*4) puts("2Mx8 ");
	if(f2bytesize==540672) puts("512Kx8 ");
	else if(f2bytesize==540672*2) puts("1Mx8 ");
	else if(f2bytesize==540672*4) puts("2Mx8 ");
	puts("\n\r");
	puts("  AVR Processor:  8MHz, 16K Flash\n\r\n\r");
	
}

//  Init AVR - TBD
int  Init_AVR()
{
	// Return 1 if AVR OK and setup or -1 if not
	return 1;
}

//  Setup Blackfin and initialize all peripherals
void Init_Blackfin()
{
	//Initialize System Configuration Register
//	sysreg_write(reg_SYSCFG, 0x32);	
	
	//  Initialize periperals and system components	
	//  Note:  For Flashed versions do not reinitialize PLL, SDRAM and Timers
	Init_PLL(); 
	Init_SDRAM(); 
	Init_Timers();
	//  Start code again
	Init_EBIU();       //  Activate APS12 Ethernet Adapter CS
//	Init_PPI_CM64();   //  Make sure Video Port PPI and PF8-15 are inputs
	Init_UART(57600);
//	Init_UART(9600);
		
	Init_Interrupts();
	Init_RealTimeClock();
	Init_MasterSPI();

	// Watchdog timer setup
	InitWatchdog();
	
    // Setup constants and variables
	programmode = LBlackfinSDRAM;
	
	//  Output startup message on UART port
	SignOnMessage();
 	
 	//  Output Ethernet and related configuration information
//	InternetSettings();
	
	//  Set Intel Hex extended address variable to zero and 
	//  initialize Flash programming parameters - SPI Master
	//  Initialize Intel hex download parameters for Flash loading
	ResetFlashVars();
	intelhexecho = 0x01;
	intelhexprintmode = LONGMODE;
	Init_Flash();
	
	// Check Flash Type(s) and set size parameters
	//  512K,1M or 2M per Flash - sets the various 
	//  parameters that interact with the Flash
	CheckFlash();
	
	//  Initialize the communication protocol to allow communication
	//  with the AVR
	if(Init_AVR()==-1)  puts("AVR not communicating - TBD\n");
	
	//  Display command prompt
	//    Note:  Blackfin command prompt is '>' while AVR prompt is ':'
	putchar('>');
	
}


//--------------------------------------------------------------------------//
// Function:	main														//
//--------------------------------------------------------------------------//
void main(void)
{
	unsigned char inchar,inbyte,instrg[3];
	unsigned int invalue;
	int i;
	long j,errorcnt,period,baseaddress;
	int bignumber[2000];
	 

	Init_Blackfin(); 
	
	loopcnt = 0;
//	for(i=0; i<60; i++)  putchar('.');
	
	while(0) {
		putchar(0x55);
		Delay(200);
		putchar(0xAA);
		Delay(400);
		}
	while(1) {
		
		if((inchar=getchar())) {
			if(inchar!=':')  putchar(inchar);
			switch (inchar) {
				case 'h': case 'H':
				    SignOnMessage(); 
				    putchar('>');
					break;
				case 'e': case 'E':
				    ExtendedMessage();
				    putchar('>');
					break;
				case 's': case 'S':
				    Specifications();
				    putchar('>');
					break;
				case 'd': case 'D': 
				    PrintMemory();
				    putchar('>');
					break;
				case 'l': case 'L':
				    LoadMemory();
				    putchar('>');
					break;
				case 'k': case 'K':
				    CompareMemory();
				    putchar('>');
					break;
				case 'b': case 'B':
				    BootApplication();
				    putchar('>');
					break;
				case 'f': case 'F':
				    FillMemoryCommand();
				    putchar('>');
					break;
				case 'm': case 'M':
				    MoveMemoryCommand();
				    putchar('>');
					break;
				case 'v': case 'V':
				    ProgramVersion();
				    putchar('>');
					break;
				case ':':
//					Delay(30000);
				    ProgramMemory();
//				    if(endofintelhex==YES)  putchar('>');
					break;
				case 'i': case 'I':
				    InternetSettings(); 
				    putchar('>');
					break;
				case 'c': case 'C':
				    Get_DateTime();
				    puts("\n\rCurrent date/time: ");
				    printdatetime();
				    puts("\n\r  Enter new [mm/dd/yr hh:mm:ss]: ");
				    Enter_DateTime();
				    puts("\n\r"); 
				    putchar('>');
					break;
				case 't': case 'T':
				    TestBlackfin(); 
					putchar('>');
					break;
				case 'p': case 'P':
				    Preferences(); 
					putchar('>');
					break;
				case 'q': case 'Q':
				    while(!(inchar=getcharnw())) {
				    	Delay(1000000);
				    	Get_DateTime();
				    	puts("\n\rCurrent date/time: ");
				    	printdatetime();
				    	}
				    puts("\n\r>"); 
					break;
				case 'x': case 'X':
				    GetInternetSettings();
				    putchar('>');
					break;
				case 'g':
				    puts("\n\rDemo I2C\n\r");
					Init_I2C();
					Set_SDA(HIGH); 
					Set_SCL(LOW);
				    while(!(inchar=getcharnw())) { 
				    	Pulse_SCL();
				    	Set_SDA(LOW);
				    	Pulse_SCL();
				    	Set_SDA(HIGH);
				    	}
				    Set_SDA(HIGH);
				    Set_SCL(HIGH);
				    puts(" Exit demo\n\r");
				    putchar('>');
					break;
				case '1':
				    puts("\n\rWrite byte to CM64\n\r");
				    while(!(inchar=getcharnw())) { 
				    //	Write_CM64(0x01,0xAA);
				    //	Init_CM64();
				    	Delay(30);
						I2C_Start();			   			// Set I2C start condition
					 	Write_I2C_Control(SAA7113_Write_ADD);	// Send the SAA7113 control Byte	
					 	I2C_Stop();	  					    // Set I2C Stop condition
 					    }
				    puts(" Exit write\n\r");
				    putchar('>');
					break;
				case '2':
				    puts("\n\rInitialize CM64 [c-color,n-no color]: ");
					inchar = getchar();
					putchar(inchar);
					// By setting the PPI to skip either even or odd bytes luminance or chromance only
					//  can be read in
					if(inchar=='c')  Init_CM64(CM64_VIDEOCHANNEL1,NOSKIP);
					else Init_CM64(CM64_VIDEOCHANNEL1,SKIPEVEN);
				    puts("\n\r>");			    
					break;
				case '3':
				    puts("\n\rHalt Video DMA/PPI operation");
				    Halt_VideoDMAPPI();
				    baseaddress = 0x0027e900;
//				    CreateBox(100,100,baseaddress);
				    puts("\n\r>");
					break;
				case '9':
				    puts("\n\rSAA7113 Status: ");
				    //Init_I2C();
				    //Reset_SAA7113();
				    inchar = Read_CM64(CM64_STATUSBYTE);
				    PrintHexByte(inchar);
				    puts("->");
				    if(inchar&0x80) puts(" Interlaced,"); else puts(" Non-interlaced,");
				    if(inchar&0x40) puts(" Not-locked,"); else puts(" Locked,");
				    if(inchar&0x20) puts(" NTSC,"); else puts(" PAL,");
				    if(inchar&0x01) puts(" Color Signal detected"); else puts(" No color signal");
				     
				    puts("\n\r");
				    putchar('>');
					break;
				case '0':
				    puts("\n\rFlash Status Word=");
				    while(!(inchar=getcharnw())) { 
				        inbyte = FlashStatus(1);
				    	}
				    PrintHexByte(inbyte);
				    if((inbyte&FLASHIDMASK)==AT45DB041ID) puts(" AT45DB041B");
				    puts("\n\r");
				    putchar('>');
					break;
				case '8':
				    puts("\n\rMaster SPI Transmit");
				    while(!(inchar=getcharnw())) {
//				    	SPI_MasterTransmit(0x55);
//						SPI_StartReadFlashPage(0);
//						SPI_ReadFlashBuffer(0,1);
	 					*pSPI_FLG = 0xFB04;
	 					Delay(1);
						SPI_ReadFlashBuffer(0,1);
	         			inbyte = SPI_MasterReceive();
	         			inbyte = SPI_MasterReceive();
		 				// Set PF2 high - deselect Flash chip
		 				Delay(1);
		 				*pSPI_FLG = 0xFF04;

//				    	bytetohex(inbyte);
//				    	puts(hexbyte);
				    	Delay(3);
				    	if(1) {
				    	    //putchar('+');
				    	    // Flash CS low	 
	 					    *pSPI_FLG = 0xFB04;
	 					    Delay(1);	 
	 					    //  Buffer write
	 					    SPI_MasterTransmit(0x84);
	 					    //putchar('-');
	 					    // Convert address to 24 bit address
	 					    ConvertFlashAddress(0);
	 					    FlashAddress[0] = 0x00;
	 					    for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 					    SPI_MasterTransmit(0x55);
	 						Delay(1);
	 					    // Set PF2 high - deselect Flash chip
	 					    *pSPI_FLG = 0xFF04;
	 					    Delay(4);
				    	    }
				    	}
				    puts("\n\r");
				    putchar('>');
					break;
				case '5':
				    puts("\n\rRead SAA7113 Chip ID continuously ");
				    Init_I2C();
				    Reset_SAA7113();
				    while(!(inchar=getcharnw())) { 
				    	inbyte = Read_CM64(CM64_CHIPVERSON);
				    	Delay(100);
				    	}
				    bytetohex(inbyte);
				    puts(hexbyte);
				    puts("\n\r");
				    putchar('>');
					break;
				case '6':
				    puts("\n\rRead SAA7113 Status continuously ");
				    Init_I2C();
				    Reset_SAA7113();
				    while(!(inchar=getcharnw())) { 
				    	//inbyte = Read_CM64(CM64_INCREMENTDELAY);
						inbyte = Read_CM64(CM64_STATUSBYTE);			    	
						Delay(100);
				    	}
				    bytetohex(inbyte);
				    puts(hexbyte);
				    puts("\n\r");
				    putchar('>');
					break;
				case '7':
				    puts("\n\rReset SAA7113 continuously ");
				    Init_I2C();
				    while(!(inchar=getcharnw())) { 
				    	Reset_SAA7113();
				    	Delay(30);
				    	}
				    puts("\n\r");
				    putchar('>');
					break;
				case '4':
				    CountPeriodStart();
				    puts("\n\rConvert 720x480 pixel image to [r-RGB, b-BW, s]: ");
				    inchar = getchar();
				    putchar(inchar);
				    
				    // Convert YUV image to RGB image - un-optimized
				    
				    if(inchar=='r') YUVtoRGB(720,488);
				    else if(inchar=='b') YUVtoGREY(720,488);
				    else CreateBox(100,100,0x0);
				    
				    puts("\n\r  Time to convert took ");
				    period = CountPeriod()/1000;
//				    period *= 8.4;
				    printint(period,5,' ');
				    puts(" msec\n\r>");
					break;
				case 'j': case 'J':
					errorcnt = 0;
					puts("\n\rCycle through 200,000 int assignments...");
				    for(j=0; j<2000; j++) {
				    	bignumber[j] = 12;
				    	if(bignumber[j] != 12) errorcnt++;
				    	}
				    if(errorcnt > 1)  puts("Error cnt > 1\n\r");
				    else puts(" no errors\n\r");
				    putchar('>');
					break;
				default:
					if(inchar==CR)  puts("\n\r");
					else if(inchar!=LF) puts("\n\rUnknown command\n\r");
					if(endofintelhex==YES)  putchar('>');
					break;
				}				

		
			
			}
			
		}
}

