// **************************************************************************
//
//  File:  P1_Video.c				Version: 0.51
//
//  Author:  Stephen Swift  		Date:  October 20, 2004		
//
//  Video interface and conversion routines for P1 video interface adapters.
//  Basic library functions are also included.
//
//  Copyright 2004, SOC Machines, Inc.
//
// **************************************************************************
#include "P1_Main.h"


// *********************************************************************
//
//  YUV (YCbCr) to RGB Conversion
//
//  Convert SAA7113 ITU656 output YUV 4:2:2 to RGB.
//
//  Two versions - use previous CbCr values for odd pixels
//               - average previous and next CbCr values for odd pixels
//
//  Variables:  numrowpixels - number of pixels
//		yuvbufptr - beginning of YUV buffer (32 bit pointer)
//		rgbbufptr - beginning of RGB buffer (32 bit pointer)
//              Pixels stored in 4:2:2 format
//
// **********************************************************************
void YUVtoRGB(short numrowpixels,short numrows)
{
	unsigned char red,green,blue,Y,Cb,Cr;
	short i,j,rowlength;
	long dest,field1,field2;

	i = j = dest = field1 = 0;
	rowlength = numrowpixels * 2;
	field2 = rowlength * (numrows/2);
	for(i=0; i<numrows; i++) {
		j = 0;

		while(j<rowlength) {
	    	// Every fourth byte is a single Y pixel without Cb, Cr 
	    	//   - use previous Cb Cr values

		    if(i%2) {
		    	Cb = *(pYUVBufPtr+field2++);
	        	Y  = *(pYUVBufPtr+field2++);
	        	Cr = *(pYUVBufPtr+field2++);
		    	}
	        else {
	        	Cb = *(pYUVBufPtr+field1++);
	        	Y  = *(pYUVBufPtr+field1++);
	        	Cr = *(pYUVBufPtr+field1++);
	        	}
	        j += 3;
		    		
			// Float version
			if(1) {
	        	red = Y + 1.371*(Cr-128);
	        	green = Y - 0.698*(Cr-128) + 0.336*(Cb-128);
  	        	blue = Y + 1.732*(Cb-128);
				}
  	    	// Integer version
  	    	else {
	        	red = Y + 0x34*(Cr-128);
	        	green = Y - 0x34*(Cr-128) + 0x23*(Cb-128);
  	        	blue = Y + 0x34*(Cb-128);
  	    		}
  	    	
			// Store results  	    
			*(pRGBBufPtr+dest++) = red;
	    	*(pRGBBufPtr+dest++) = green;
	    	*(pRGBBufPtr+dest++) = blue;
				  	
	    	//  Do isolated luminance
	    	if(i%2) Y = *(pYUVBufPtr+field2++);
	    	else Y = *(pYUVBufPtr+field1++);
		    j++;
		    
		    			// Float version
			if(1) {
	        	red = Y + 1.371*(Cr-128);
	        	green = Y - 0.698*(Cr-128) + 0.336*(Cb-128);
  	        	blue = Y + 1.732*(Cb-128);
				}
  	    	// Integer version
  	    	else {
	        	red = Y + 0x34*(Cr-128);
	        	green = Y - 0x34*(Cr-128) + 0x23*(Cb-128);
  	        	blue = Y + 0x34*(Cb-128);
  	    		}
  	    	
			// Store results  
			*(pRGBBufPtr+dest++) = red;
	    	*(pRGBBufPtr+dest++) = green;
	    	*(pRGBBufPtr+dest++) = blue;	
	    	}
		}
}

// *********************************************************************
//
//  YUV (YCbCr) to Grey Scale
//
//  Convert SAA7113 ITU656 output YUV 4:2:2 to Grey scale.
//
//  Variables:  numrowpixels - number of pixels
//		yuvbufptr - beginning of YUV buffer (32 bit pointer)
//		rgbbufptr - beginning of RGB buffer (32 bit pointer)
//              Pixels stored in 4:2:2 format
//
// **********************************************************************
void YUVtoGREY(short numrowpixels,short numrows)
{
	unsigned char red,green,blue,Y,Cb,Cr;
	short i,j;
	long k,field1,field2;

	k = field1 = 0;
	field2 = numrowpixels * numrows/2;
	for(i=0; i<numrows; i++) {
		for(j=0; j<numrowpixels; j++) {
			// Get luminance
			if(i%2) Y = *(pYUVBufPtr+field2++); 
			else  Y = *(pYUVBufPtr+field1++); 	
			    	
			// Store results  	    
	    	*(pRGBBufPtr+k++) = Y;
	    	}
		}
}

// *********************************************************************
//
//  RGB to YUV (YCbCr)  Conversion
//
//  Convert RGB to SAA7113 ITU656 output YUV 4:2:2.
//
//  Two versions - use previous CbCr values for odd pixels
//               - average previous and next CbCr values for odd pixels
//
// **********************************************************************
void RGBtoYUV()
{
}

// *********************************************************************
//
//  Create Box for row, column size
//
//  Convert SAA7113 ITU656 output YUV 4:2:2 to RGB.
//
//  Two versions - use previous CbCr values for odd pixels
//               - average previous and next CbCr values for odd pixels
//
//  Variables:  numrowpixels - number of pixels
//		yuvbufptr - beginning of YUV buffer (32 bit pointer)
//		rgbbufptr - beginning of RGB buffer (32 bit pointer)
//              Pixels stored in 4:2:2 format
//
// **********************************************************************
void CreateBox(short pixelsonrow,short numrows,long baseaddress)
{
	unsigned char red,green,blue;
	short i,j,rowlength;
	long dest;

	dest = 0;
	for(i=0; i<numrows; i++) {
		for(j=0; j<pixelsonrow; j++) {
			*(pRGBBufPtr+baseaddress+dest++) = 0xff;
	    	*(pRGBBufPtr+baseaddress+dest++) = 0xff;
	    	*(pRGBBufPtr+baseaddress+dest++) = 0xff;
			}
		}
	
	// top row
	dest = 0;
	
	for(i=0; i<pixelsonrow; i++) {
		*(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
		}
	for(i=0; i<pixelsonrow; i++) {
		*(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
		}
	for(i=0; i<pixelsonrow; i++) {
		*(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
		}
	
	// Bottom row
	dest = pixelsonrow*3*(numrows-3);
	for(i=0; i<pixelsonrow; i++) {
		*(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
		}
	for(i=0; i<pixelsonrow; i++) {
		*(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
		}
	for(i=0; i<pixelsonrow; i++) {
		*(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
	    *(pRGBBufPtr+baseaddress+dest++) = 0x35;
		}

}

