/*****************************************************************************
**																			**
**	 Name: 	P1 Pirana Boot Monitor										    **	
**																			**
******************************************************************************

Project Name:	P1 Blackfin Monitor.c 

Date Modified:	10/3/08		SS		Ver 0.71

Software:		VisualDSP++ 4.5

Hardware:		P1 Pirana Embedded Video Processor

Purpose:		Implements a basic Monitor for the P1 Pirana Embedded video 
				processor. Code is provided to program the CM64 Video Adapter, 
				Serial port.
				
				TODO - add rudimentary OS for eBoot support
				       and/or uCLinux boot support

*/
/*
 * Copyright (C) 2005-2008 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */
/*
 * $Log: P1_Blackfin_Monitor.c,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
#include "P1_Main.h"
#include "ccblkfn.h"
#include "sysreg.h"

#include "P1_I2C.h"
#include "SAA7113.h"  
#include "math.h"
#include "P1_Extern.h"

//--------------------------------------------------------------------------//
// Variables																//
//--------------------------------------------------------------------------//

//  Variable declarations
unsigned char programmode,recordtype;
unsigned char ip[4],ipaddress[4],ipmask[4],ipgateway[4],ethernetmac[6];
unsigned int bytesonline,inteladdress,extendedaddress;
unsigned char crc,intelbinary[40];  // length can vary
char hexbyte[3],hexaddress[5],SPI_State,endofintelhex,intelhexecho,intelhexprintmode;
char daytab[2][13] = {
	{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
	{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
	};
unsigned char FlashAddress[3];
int second,minute,hour,day,year;
int f1pagesize,f2pagesize,fillindex,loopcnt;
unsigned long addressptr;
long f1bytesize,f2bytesize;

//  Flash initialization and control paramters
int currentflashbyteoffset;  // Current byte in flash RAM buffer
int currentflashpage;   // Current flash page being loaded
int currentflashchip;   // Current flashchip being accessed - 1 or 2
char flashbufferloaded;  // Status of flash buffer load - YES/NO
char flashpageprogrammed; // Status of flash page programming - YES/NO
char flashbufferempty;   // True if data loaded into flash RAM buffer but not programmed


/* Static Variables */
unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
unsigned char UART_RxHead;
unsigned char UART_RxTail;
unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
unsigned char UART_TxHead;
unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars, sendxonxoff;
char  rxcnt, checkflag;

//--------------------------------------------------------------------------//
// Function:	main														//
//--------------------------------------------------------------------------//
void main(void)
{
	unsigned char inchar,inbyte,instrg[3];
	char  strg[100],inbyte2;
	unsigned int invalue;
	int i,num1,num2,num3,inword;
	short channel;
	long j,errorcnt,period,baseaddress,startaddress;
	float floatnum;
	 
	Init_Blackfin();  
	
	loopcnt = 0;
//	for(i=0; i<60; i++)  putchar('.');

			
	while(1) {
		
		//printf("\n\rHello world - enter 3 integers: ");
		//scanf("%d %d %d",&num1,&num2,&num3);
		//printf("\n\rnumbers = %d %d %d\n\r>",num1,num2,num3);
		//printf("\n\rEnter num string: ");
		//scanf("%d %s",&num1,strg);
		//printf("\n\rstring = %d %s\n\r>",num1,strg);
		
		//printf("\n\rEnter float number: ");
		//scanf("%f",&floatnum);
		//printf("\n\rFloat num=%f\n\r",floatnum);

		if((inchar=getchar())) {
			if((inchar!=':')&&(inchar!='a'))  putchar(inchar);
			switch (inchar) {
				case 'h': case 'H':
				    SignOnMessage(); 
				    putchar('>');
					break;
				case 'a': case 'A':
				    CapturePicture();
				    putchar('>');
				    //SpartanTest();
					break;
				case 'e': case 'E':
				    ExtendedMessage();
				    putchar('>');
					break;
				case 's': case 'S':
				    Specifications();
				    putchar('>');
					break;
				case 'd': case 'D': 
				    PrintMemory();
				    putchar('>');
					break;
				case 'l': case 'L':
				    LoadMemory();
				    putchar('>');
					break;
				case 'k': case 'K':
				    CompareMemory();
				    putchar('>');
					break;
				case 'b': case 'B':
				    BootApplication();
				    putchar('>');
					break;
				case 'f': case 'F':
				    FillMemoryCommand();
				    putchar('>');
					break;
				case 'm': case 'M':
				    MoveMemoryCommand();
				    putchar('>');
					break;
				case 'v': case 'V':
				    ProgramVersion();
				    putchar('>');
					break;
				case ':':
//					Delay(30000);
				    ProgramMemory();
//				    if(endofintelhex==YES)  putchar('>');
					break;
				case 'i': case 'I':
				    InternetSettings(); 
				    putchar('>');
					break;
				case 'c': case 'C':
				    Get_DateTime();
				    puts("\n\rCurrent date/time: ");
				    printdatetime();
				    puts("\n\r  Enter new [mm/dd/yr hh:mm:ss]: ");
				    Enter_DateTime();
				    puts("\n\r"); 
				    putchar('>');
					break;
				case 't': case 'T':
				    TestBlackfin(); 
					putchar('>');
					break;
				case 'p': case 'P':
				    Preferences(); 
					putchar('>');
					break;
				case 'q': case 'Q':
				    while(!(inchar=getcharnw())) {
				    	Delay(1000000);
				    	Get_DateTime();
				    	puts("\n\rCurrent date/time: ");
				    	printdatetime();
				    	}
				    puts("\n\r>"); 
					break;
				case 'x': case 'X':
				    GetInternetSettings();
				    putchar('>');
					break;
/*				case 'g':
				    puts("\n\rDemo I2C\n\r");
					Init_I2C();
					Set_SDA(HIGH); 
					Set_SCL(LOW);
				    while(!(inchar=getcharnw())) { 
				    	Pulse_SCL();
				    	Set_SDA(LOW);
				    	Pulse_SCL();
				    	Set_SDA(HIGH);
				    	}
				    Set_SDA(HIGH);
				    Set_SCL(HIGH);
				    puts(" Exit demo\n\r");
				    putchar('>');
					break;
				case '1':
				    puts("\n\rWrite byte to CM64\n\r");
				    while(!(inchar=getcharnw())) { 
				    //	Write_CM64(0x01,0xAA);
				    //	Init_CM64();
				    	Delay(30);
						I2C_Start();			   			// Set I2C start condition
					 	Write_I2C_Control(SAA7113_Write_ADD);	// Send the SAA7113 control Byte	
					 	I2C_Stop();	  					    // Set I2C Stop condition
 					    }
				    puts(" Exit write\n\r");
				    putchar('>');
					break;
*/
				case '2':
				      puts("\n\rInitialize CM64 [c-color,n-no color]: ");
					  inchar = getchar();
					  putchar(inchar);
					  puts("\n\rSelect Video Input Channel [0,1,2,3]: ");
					  channel = getint();
					  putchar(channel);
					  puts("\n\rStart address (hex): ");
					  startaddress = getlonghex();
					  // By setting the PPI to skip either even or odd bytes luminance or chromance only
					  //  can be read in
					  if(inchar=='c')  Init_CM64(channel,NOSKIP,startaddress);
					  else Init_CM64(channel,SKIPEVEN,startaddress);
				      if(0) {
				        puts("Dump SAA7113 registers\n\r");
				    	Init_I2C();
				    	//Reset_SAA7113();
				        for(i=0; i<0x17; i++) {
				            inchar = Read_CM64((unsigned char) i);
				            printf("%x-%x ",i,inchar);
				        	}
				        inchar = Read_CM64((unsigned char) 0x1f);
				        printf("%x-%x ",0x1f,inchar);
				        inchar = Read_CM64((unsigned char) 0x40);
				        printf("%x-%x ",0x40,inchar);
				        for(i=0x41; i<0x5c; i++) {
				            inchar = Read_CM64((unsigned char) i);
				            printf("%x-%x ",i,inchar);
				        	}
				        for(i=0x5e; i<0x62; i++) {
				            inchar = Read_CM64((unsigned char) i);
				            printf("%x-%x ",i,inchar);
				        	}
				    	}
				    puts("\n\r>");			    
					break;
				case '3':
				    puts("\n\rHalt Video DMA/PPI operation");
				    Halt_VideoDMAPPI();
				    Reset_SAA7113();
				    baseaddress = 0x0027e900;
//				    CreateBox(100,100,baseaddress);
				    puts("\n\r>");
					break;
				case '9':
				    puts("\n\rSAA7113 Status: ");
				    //Init_I2C();
				    //Reset_SAA7113();
				    inchar = Read_CM64(CM64_STATUSBYTE);
				    PrintHexByte(inchar);
				    puts("->");
				    if(inchar&0x80) puts(" Interlaced,"); else puts(" Non-interlaced,");
				    if(inchar&0x40) puts(" Not-locked,"); else puts(" Locked,");
				    if(inchar&0x20) puts(" NTSC,"); else puts(" PAL,");
				    if(inchar&0x01) puts(" Color Signal detected"); else puts(" No color signal");
				     
				    puts("\n\r");
				    putchar('>');
					break;
				case '0':
				    puts("\n\rFlash Status Word=");
				    while(!(inchar=getcharnw())) { 
				        inbyte = FlashStatus(1);
				    	}
				    PrintHexByte(inbyte);
				    if((inbyte&FLASHIDMASK)==AT45DB041ID) puts(" AT45DB041B");
				    puts("\n\r");
				    putchar('>');
					break;
				case '8':
				    puts("\n\rMaster SPI Transmit");
				    while(!(inchar=getcharnw())) {
//				    	SPI_MasterTransmit(0x55);
//						SPI_StartReadFlashPage(0);
//						SPI_ReadFlashBuffer(0,1);
	 					*pSPI_FLG = 0xFB04;
	 					Delay(1);
						SPI_ReadFlashBuffer(0,1);
	         			inbyte = SPI_MasterReceive();
	         			inbyte = SPI_MasterReceive();
		 				// Set PF2 high - deselect Flash chip
		 				Delay(1);
		 				*pSPI_FLG = 0xFF04;

//				    	bytetohex(inbyte);
//				    	puts(hexbyte);
				    	Delay(3);
				    	if(1) {
				    	    //putchar('+');
				    	    // Flash CS low	 
	 					    *pSPI_FLG = 0xFB04;
	 					    Delay(1);	 
	 					    //  Buffer write
	 					    SPI_MasterTransmit(0x84);
	 					    //putchar('-');
	 					    // Convert address to 24 bit address
	 					    ConvertFlashAddress(0);
	 					   // FlashAddress[0] = 0x00;
	 					   // for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 					    SPI_MasterTransmit(0x55);
	 						Delay(1);
	 					    // Set PF2 high - deselect Flash chip
	 					    *pSPI_FLG = 0xFF04;
	 					    Delay(4);
				    	    }
				    	}
				    puts("\n\r");
				    putchar('>');
					break;
				case '5':
				    puts("\n\rRead SAA7113 Chip ID continuously ");
				    Init_I2C();
				    Reset_SAA7113();
				    while(!(inchar=getcharnw())) { 
				    	inbyte = Read_CM64(CM64_CHIPVERSON);
				    	Delay(100);
				    	}
				    bytetohex(inbyte);
				    puts(hexbyte);
				    puts("\n\r");
				    putchar('>');
					break;
				case '6':
				    puts("\n\rRead SAA7113 Status continuously ");
				    Init_I2C();
				    Reset_SAA7113();
				    while(!(inchar=getcharnw())) { 
				    	//inbyte = Read_CM64(CM64_INCREMENTDELAY);
						inbyte = Read_CM64(CM64_STATUSBYTE);			    	
						Delay(100);
				    	}
				    bytetohex(inbyte);
				    puts(hexbyte);
				    puts("\n\r");
				    putchar('>');
					break;
				case '7':
				    puts("\n\rReset SAA7113 continuously ");
				    Init_I2C();
				    while(!(inchar=getcharnw())) { 
				    	Reset_SAA7113();
				    	Delay(30);
				    	}
				    puts("\n\r");
				    putchar('>');
					break;
/*				case '4':
				    puts("\n\rConvert 720x480 pixel image to [r-RGB,b-BW,s,c,d]: ");
				    inchar = getchar();
				    putchar(inchar);
				    CountPeriodStart();
				    
				    // Convert YUV image to RGB image - un-optimized
				    
				    if(inchar=='r') {
				    	CountPeriodStart();
				    	YUVtoRGB(360,480);
				    	}
				    else if(inchar=='b') {
				    	CountPeriodStart();
				    	YUVtoGREY(360,480);
				    	}
				    else if(inchar=='c') {
				    	puts("\n\rEnter color: ");
				    	inchar = getlonghex();
				    	putchar(inchar);
				    	CountPeriodStart();
				    	CreateColor(360,480,0x0,inchar);
				    	}
				    else if(inchar=='d') {
				    	puts("\n\rEnter color: ");
				    	inword = getlonghex();
				    	puts("\n\rcolor: ");
				    	printlonghex(inword);
				    	CountPeriodStart();
				    	CreateColorW(360,480,0x0,inword);
				    	}
				    else {
				    	CountPeriodStart();
				    	CreateBox(200,200,0x0);
				    	}
				    
				    period = CountPeriod()/1000;
				    puts("\n\r  Time to convert: ");
				    period *= 8.4;
				    printint(period,5,' ');
				    puts(" msec\n\r>");
					break;
*/
				case 'j': case 'J':
					GCodeTest();
				    //putchar('>');
					break;
				default:
					if(inchar==CR)  puts("\n\r");
					else if(inchar!=LF) puts("\n\rUnknown command\n\r");
					if(endofintelhex==YES)  putchar('>');
					break;
				}				

		
			
			}
			
		}
}

