// *******************************************************
//
//   Program:  P1_Main.c        Version: 0.72 
//
//   Author: Stephen Swift		Date:  Jan 20, 2009
//
//   Main include file for P1 Blackfin Monitor Program.
//
// *******************************************************
/*
 * Copyright (C) 2005-2006 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Main.h,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
 #ifndef  __P1_FLAGS_DEFINED
#define __P1_FLAGS_DEFINED

//--------------------------------------------------------------------------//
// Header files																//
//--------------------------------------------------------------------------//
#include <sys\exception.h>
#include <cdefBF532.h>
#include "stdarg.h"
#include "string.h"
//#include "stdio.h"

//--------------------------------------------------------------------------//
// Symbolic constants														//
//--------------------------------------------------------------------------//

/* USART Buffer Defines */
#define 	UART_RX_BUFFER_SIZE 128 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define 	UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#define 	UART_TX_BUFFER_SIZE 128 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define 	UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )

#define 	NO  		0x00
#define 	YES 		0x01
#define 	ON  		0x00
#define 	OFF 		0x01
#define 	XON   		0x11
#define 	XOFF  		0x13
#define  	SPACE 	   	0x20 
#define		ACK			0x06
#define		BS			0x08
#define		TAB			0x09
#define		CR			0x0D
#define		LF			0x0A
#define		ESC			0x1B

//  General Definitions
#define		LBlackfinSDRAM	's'
#define		LFLASH			'f'
#define		READCONTINUOUS	1
#define		READBUFFER		2
#define		INTELHEXBYTESPERLINE	16
#define		PRINTINTELHEXASCII	'p'	
#define		SHORTMODE		'S'
#define		LONGMODE		'L'
// For 512Kx8 Flash	
#define     FLASHMAXLENGTH 	0x80000
// For 1Mx8 Flash
//#define 	FLASHMAXLENGTH	0x100000
// For 2Mx8 Flash
//#define	FLASHMAXLENGTH	0x200000
#define		NOFLASHCHIPS	1
// Set to 264 for AT45DB041/081 chips and 528 for AT45DB161 chips
#define		FLASHBYTESPERPAGE 264
#define		HIGH		1
#define		LOW			2
#define		AT45DB041ID  0x1C
#define		AT45DB081ID  0x24
#define		AT45DB161ID  0x2C
#define		AT45DB321ID  0x34
#define		AT45DB642ID  0x3C
#define 	FLASHIDMASK	 0x3C

// Video Global Definitions
// Start address of YUV Buffer filled by DMA from CM64
#define 	YUVBufPtr   0x00000000
#define 	pYUVBufPtr ((volatile unsigned char *)YUVBufPtr)
// Start address of RGB Buffer - 2,097,152 byte offset from 0
#define 	RGBBufPtr   0x00200000
#define 	pRGBBufPtr ((volatile unsigned char *)RGBBufPtr)

#define 	pwYUVBufPtr ((volatile unsigned short *)YUVBufPtr)
#define 	pwRGBBufPtr ((volatile unsigned short *)RGBBufPtr)

// SDRAM Range Settings
#define 	SDRAMBot   0x00000000
#define 	pSDRAMBot ((volatile unsigned char *)SDRAMBot)
#define 	SDRAMTop   0x01FFFFFF
#define 	pSDRAMTop ((volatile unsigned char *)SDRAMTop)
#define 	SDRAMWordBot   0x00000000
#define 	pSDRAMWordBot ((volatile unsigned short *)SDRAMBot)

// Chip Select Selections
#define 	AMS3	0x20300000
#define 	pAMS3 ((volatile unsigned short *)AMS3)
#define 	AMS2	0x20200000
#define 	pAMS2 ((volatile unsigned short *)AMS2)
#define 	AMS1	0x20100000
#define 	pAMS1 ((volatile unsigned short *)AMS1)
#define 	AMS0	0x20000000
#define 	pAMS0 ((volatile unsigned short *)AMS0)

#define 	AMS2B	0x20200000
#define 	pAMS2B ((volatile unsigned char *)AMS3B)
#define 	AMS3B	0x20300000
#define 	pAMS3B ((volatile unsigned char *)AMS3B)

//  EEPROM Definitions
#define   MAJORVER   	 0
#define   MINORVER       72
#define   PROCESSORTYPE  70  // P1 processor
#define   ETHERNETMACID  "002078e11156"
#define   IPADDRESS      "256.128.0.24"
#define   IPMASK         "256.256.256.0"
#define   IPGATEWAY		 "256.45.25.0"
#define   USERID		 "PIRANA1"
#define   PASSWORD       "SECRET"

//  Other constants
#define	  FROMSDRAM		0x02
#define	  FROMFLASH     0x01
#define   FROMCONST     0x03

//  Declarations
#define RAM_Length 189000
#define CLKIN 	27

//--------------------------------------------------------------------------//
// Global variables															//
//--------------------------------------------------------------------------//
extern short sLight_Move_Direction;

// In file ISRs.c
EX_INTERRUPT_HANDLER(Timer0_ISR);
EX_INTERRUPT_HANDLER(DMA0_PPI_ISR);		// assign DMA0 PPI ISR to interrupt vector 8
EX_INTERRUPT_HANDLER(UART_RX_ISR);		// UART Rx ISR -> IVG 9	
EX_INTERRUPT_HANDLER(UART_TX_ISR);
#endif //__P1_FLAGS_DEFINED



