// **************************************************************
//
//  File:  SAA7113.h			Version: 0.45
//
//  Date:  October 20, 2008		
//
//  SAA7113 Definitions for CM64 NTSC/PAL Analog Camera Module
//
//  Copyright 2004-2008, SOC Robotics, Inc.  All rights reserved.
//
// **************************************************************
/*
 * Copyright (C) 2005-2006 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: SAA7113.h.c,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
//  Constants
#define  CM64_VIDEOCHANNEL1			0x00
#define  CM64_VIDEOCHANNEL2			0x01
#define  CM64_VIDEOCHANNEL3			0x02
#define  CM64_VIDEOCHANNEL4			0x03

//  Sub-Address Register Definitions
//
//   Read only:
//		CM64_CHIPVERSON
//		CM64_STATUSBYTE
//		CM64_DR
//		CM64_LN1
//		CM64_LN2

#define   CM64_CHIPVERSON			0x00
#define   CM64_INCREMENTDELAY		0x01
#define   CM64_ANINPUTCNTL1			0x02
#define   CM64_ANINPUTCNTL2			0x03
#define   CM64_ANINPUTCNTL3			0x04
#define   CM64_ANINPUTCNTL4			0x05
#define   CM64_HSYNCSTART			0x06
#define   CM64_HSYNCSTOP			0x07
#define   CM64_SYNCCNTL				0x08
#define   CM64_LUMINANCECNTL		0x09
#define   CM64_LUMINANCEBRIGHTNESS  0x0A
#define   CM64_LUMINANCECONTRAST	0x0B
#define   CM64_CHROMASATURATION		0x0C
#define   CM64_CHROMAHUECNTL		0x0D
#define   CM64_CHROMACNTL			0x0E
#define   CM64_CHROMAGAINCNTL		0x0F
#define   CM64_FORMATDELAYCNTL		0x10
#define   CM64_OUTCNTL1				0x11
#define   CM64_OUTCNTL2				0x12
#define   CM64_OUTCNTL3				0x13
#define   CM64_VGATE1START			0x15
#define   CM64_VGATE1STOP			0x16
#define   CM64_VGATE1MSB			0x17
#define   CM64_STATUSBYTE			0X1F
#define   CM64_AC1					0x40
#define   CM64_LCR2					0x41
#define   CM64_LCR3					0x42
#define   CM64_LCR4					0x43
#define   CM64_LCR5					0x44
#define   CM64_LCR6					0x45
#define   CM64_LCR7					0x46
#define   CM64_LCR8					0x47
#define   CM64_LCR9					0x48
#define   CM64_LCR10				0x49
#define   CM64_LCR11				0x4A
#define   CM64_LCR12				0x4B
#define   CM64_LCR13				0x4C
#define   CM64_LCR14				0x4D
#define   CM64_LCR15				0x4E
#define   CM64_LCR16				0x4F
#define   CM64_LCR17				0x50
#define   CM64_LCR18				0x51
#define   CM64_LCR19				0x52
#define   CM64_LCR20				0x53
#define   CM64_LCR21				0x54
#define   CM64_LCR22				0x55
#define   CM64_LCR23				0x56
#define   CM64_LCR24				0x57
#define   CM64_FC					0x58
#define   CM64_HOFF					0x59
#define   CM64_VOFF					0x5A
#define   CM64_HVOFF				0x5B
#define   CM64_SDID					0x5E
#define   CM64_DR					0x60
#define   CM64_LN1					0x61
#define   CM64_LN2					0x62

//  Register Settings

//  SubAddress 02H - Analog Cntl 1  Update hysteresis D5,4 - Analog function D7,6
#define	  SAA_MODE_CH11		0x00
#define	  SAA_MODE_CH12		0x01
#define	  SAA_MODE_CH21		0x02
#define	  SAA_MODE_CH22		0x03
//  SubAddress 03H - Analog Cntl 2  
//		D0 - Sign bit gain control channel 1
//		D1 - Sign bit gain control channel 2
//      D2 - Gain control fix - 1 user set gain
//      D3 - AGC control - 1 gain fixed
//      D4 - White peak - 1 0ff
//      D5 - AGC hold during vertical blanking - 1 AGC disbled
//      D6 - HL not referenced ?

//  SubAddress 04H/05H - manual gain setting

//  SubAddress 06H - Horizontal sync beings
//     -108 (50Hz) - 10010100B
//     -107 (60Hz) - 10010101B
//      108 (50Hz) - 01101100B
//      107 (60Hz) - 01101011B
//      Raw Data   - 11101001B

//  SubAddress 07H - Horizontal sync stops (see above)
//      Raw Data   - 00001101B

//  SubAddress 08H - Vertical noise reduction
//      recommended setting 0x90

//  SubAddress 09H - Luminance control
//      See manual

//  SubAddress 0AH - Luminance brightness
//      128 - CCIR level  0x80
#define   SAA_LUMINANCEBRIGHTNESS  0x80

//  SubAddress 0BH - Luminance contrast
//      1.109 - CCIR level  0x40
#define   SAA_LUMINANCECONTRAST    0x40

//  SubAddress 0CH - Chrominance Saturation
//      1.0 - CCIR level  0x40
#define   SAA_CHROMINANCESAT	   0x40

//  SubAddress 0DH - Chrominance Hue
//      0 - 0
#define   SAA_CHROMINANCEHUE       0x00

//  SubAddress 0EH - Chrominance Control
//    Bandwidth
#define   SAA_BANDWIDTH_620KHZ		0x00
#define   SAA_BANDWIDTH_800KHZ		0x01
#define   SAA_BANDWIDTH_920KHZ		0x02
#define   SAA_BANDWIDTH_1000KHZ		0x03
//    Colour time
#define   SAA_COLOUR_NOMINAL		0x00
#define   SAA_COLOUR_FAST			0x04
//    Chroma comb filter
#define   SAA_CHROMACOMB_ON			0x00
#define   SAA_CHROMACOMB_OFF		0x08
//    Colour Standard
#define   SAA_NTSC_M		0x00
#define   SAA_PAL4-43		0x10
#define   SAA_NTSC4-43		0x20
#define   SAA_PALM			0x30

//  SubAddress 0FH - Chrominance Gain Control
//    Set by D6-D0 (Only if ACGC is set to 1)
#define   SAA_AGCSWITCHOFF		0x80

//  SubAddress 10H - Format/Delay Control
//     Luminance delay D2-0
//     VREF position   D3
//     HS Fine position D5,4
#define   SAA_VFLAG_ITU656		0x00
#define   SAA_VFLAG_VREF		0x40
#define   SAA_VFLAG_DATATYPE	0x80

//  SubAddress 11H - Output Control 1
#define   SAA_COLOUR_ON			0x01
#define   SAA_YUV_BYPASS		0x02
#define   SAA_OUTPUT_ENABLE		0x04
#define   SAA_OUTPUT_VIDEO		0x08
#define   SAA_FASTLOCK			0x10
#define   SAA_RTSO_HIGH			0x20
#define   SAA_RTS1_HIGH			0x80

//  SubAddress 12H - Output Control 2
//     See manual page 63
//     TBD - default setting
#define   SAA_AOUT_AD1			0x01
#define   SAA_AOUT_AD2			0x02
#define   SAA_FID_INVERTED		0x04

//  SubAddress 15H - VGATE Start Pulse
//		See manual page 66 - default to 0x00

//  SubAddress 16H - VGATE Stop Pulse
//      See manual page 67 - default to 0x00

//  SubAddress 17H - VGATE MSBs
//      See manual page 68

//  SubAddress 1FH - Status byte video decoder
//    CODE = 1 - video sync'd
//    INTL = 1 - video interlaced
#define	  SAA_RDCAP		0x00
#define   SAA_CODE		0x01
#define   SAA_COPRO		0x02
#define   SAA_WIPA		0x04
#define   SAA_GLIMB		0x08
#define   SAA_GLIMT		0x10
#define   SAA_FIDT		0x20
#define   SAA_HLVLN		0x40
#define   SAA_INTL		0x80

//  SubAddress 40H - 
//      See manual page 68

//  SubAddresses 41H to 57H -  LCR registers
//      See manual page 70




